import os
from Bio import SeqIO


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if (each_element.isalpha() is True) or (each_element == '='):
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def get_cigar_stats(cigar_splitted):

    # aligned_len: M I X =
    # clipping_len: S
    # mismatch_len: X I D
    # mismatch_pct = mismatch_len / aligned_len
    # aligned_pct  = aligned_len  / (aligned_len + clipping_len)
    # clipping_pct = clipping_len / (aligned_len + clipping_len)

    aligned_len = 0
    clipping_len = 0
    mismatch_len = 0
    for each_part in cigar_splitted:
        each_part_len = int(each_part[:-1])
        each_part_cate = each_part[-1]

        # get aligned_len
        if each_part_cate in {'M', 'm', 'I', 'i', 'X', 'x', '='}:
            aligned_len += each_part_len

        # get clipping_len
        if each_part_cate in ['S', 's']:
            clipping_len += each_part_len

        # get mismatch_len
        if each_part_cate in {'I', 'i', 'X', 'x', 'D', 'd'}:
            mismatch_len += each_part_len

    aligned_pct  = float("{0:.2f}".format(aligned_len * 100 / (aligned_len + clipping_len)))
    clipping_pct = float("{0:.2f}".format(clipping_len * 100 / (aligned_len + clipping_len)))
    mismatch_pct = float("{0:.2f}".format(mismatch_len * 100 / (aligned_len)))

    return aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct


def check_both_ends_clipping(cigar_splitted):

    both_ends_clipping = False
    if len(cigar_splitted) >= 3:
        if (cigar_splitted[0][-1] in ['S', 's']) and (cigar_splitted[-1][-1] in ['S', 's']):
            both_ends_clipping = True

    return both_ends_clipping


def get_min_mismatch_from_cigar_list(r1_ref_cigar_set, min_M_len):

    mismatch_set_all_cigar = set()
    mismatch_set_long_M_cigars = set()
    for each_cigar in r1_ref_cigar_set:
        aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct = get_cigar_stats(cigar_splitter(each_cigar))
        mismatch_set_all_cigar.add(mismatch_pct)
        if aligned_len >= min_M_len:
            mismatch_set_long_M_cigars.add(mismatch_pct)

    min_mismatch = 'NA'
    if len(mismatch_set_all_cigar) > 0:
        min_mismatch = min(mismatch_set_all_cigar)
        if len(mismatch_set_long_M_cigars) > 0:
            min_mismatch = min(mismatch_set_long_M_cigars)

    return min_mismatch


class MappingRecord:

    def __init__(self):

        #################### overall ####################

        self.qualified_reads = False

        #################### round 1 16s ####################

        self.r1_16s_ref_dict = dict()
        self.r2_16s_ref_dict = dict()

        self.r1_16s_refs_lowest_mismatch = None
        self.r2_16s_refs_lowest_mismatch = None

        self.r1_16s_refs_no_ignored = dict()
        self.r2_16s_refs_no_ignored = dict()
        self.shared_16s_refs_no_ignored = dict()

        self.both_mapped_to_16s = False

        #################### round 1 ctg ####################

        self.r1_ctg_ref_dict = dict()
        self.r2_ctg_ref_dict = dict()

        self.r1_ctg_refs_lowest_mismatch = None
        self.r2_ctg_refs_lowest_mismatch = None

        self.r1_ctg_refs_no_ignored = dict()
        self.r2_ctg_refs_no_ignored = dict()
        self.shared_ctg_refs_no_ignored = dict()

        self.matched_to_ctg = False

        #################### round 2 ####################

        self.qualified_reads_rd2 = False

        self.r1_ctg_ref_dict_rd2 = dict()
        self.r2_ctg_ref_dict_rd2 = dict()

        self.r1_ctg_refs_lowest_mismatch_rd2 = None
        self.r2_ctg_refs_lowest_mismatch_rd2 = None

        #################################################


        self.r1_longest_clp_cigar = ''
        self.r1_longest_clp_falg = ''
        self.r2_longest_clp_cigar = ''
        self.r2_longest_clp_falg = ''

        self.consider_r1_unmapped_mate = False
        self.consider_r1_clipping_part = False
        self.consider_r2_unmapped_mate = False
        self.consider_r2_clipping_part = False

        self.consider_round_2 = False

        self.r1_filtered_refs = set()
        self.r2_filtered_refs = set()
        self.unmapped_r1_refs = set()
        self.unmapped_r2_refs = set()
        self.clipping_r1_refs = set()
        self.clipping_r2_refs = set()
        self.unmapped_r1_refs_with_pos = set()
        self.unmapped_r2_refs_with_pos = set()
        self.clipping_r1_refs_with_pos = set()
        self.clipping_r2_refs_with_pos = set()


####################################################################################################################

rd1_unlinked_mags_sam_bowtie_reformat = '/Users/songweizhi/Desktop/tunning_rd2/round_1_unlinked_gnm_bowtie_reformatted.sam'
min_M_len_ctg = 30
mismatch_cutoff = 2
report_interval = 100000
'''
# should keep both, but only kept one !!!
MBARC26_13313152	{'TC___C___NODE_809_length_22129_cov_127.040953_l': {1: '66S55=1X8='}, 'TC___C___NODE_189_length_107521_cov_122.830635_l': {1: '76S45=1X8='}}
MBARC26_13313152.1	TC___C___NODE_809_length_22129_cov_127.040953_l	{1: '66S55=1X8='}
'''

reads_file_r1_fasta = ''
reads_file_r2_fasta = ''
rd2_to_extract_flking_ctg_r1_id     = 'rd2_to_extract_flking_ctg_r1_id.txt'
rd2_to_extract_flking_ctg_r2_id     = 'rd2_to_extract_flking_ctg_r2_id.txt'
rd2_extracted_flking_ctg_r1_seq_tmp = 'rd2_extracted_flking_ctg_r1_tmp.fa'
rd2_extracted_flking_ctg_r2_seq_tmp = 'rd2_extracted_flking_ctg_r2_tmp.fa'
rd2_extracted_flking_ctg_r1_seq     = 'rd2_extracted_flking_ctg_r1.fa'
rd2_extracted_flking_ctg_r2_seq     = 'rd2_extracted_flking_ctg_r2.fa'


####################################################################################################################

round_2_ctg_end_seq_len_dict = {}
round_2_MappingRecord_dict = {}
for each_line in open(rd1_unlinked_mags_sam_bowtie_reformat):
    each_line_split = each_line.strip().split('\t')
    if each_line.startswith('@'):
        rd2_ref_id = ''
        rd2_ref_len = 0
        for each_element in each_line_split:
            if each_element.startswith('SN:'):
                rd2_ref_id = each_element[3:]
            if each_element.startswith('LN:'):
                rd2_ref_len = int(each_element[3:])
        round_2_ctg_end_seq_len_dict[rd2_ref_id] = rd2_ref_len
    else:
        cigar = each_line_split[5]
        if cigar != '*':
            read_id = each_line_split[0]
            read_id_base = '.'.join(read_id.split('.')[:-1])
            read_strand = read_id.split('.')[-1]
            ref_id = each_line_split[2]
            ref_pos = int(each_line_split[3])

            if read_id_base not in round_2_MappingRecord_dict:
                round_2_MappingRecord_dict[read_id_base] = MappingRecord()

            if read_strand == '1':
                if ref_id not in round_2_MappingRecord_dict[read_id_base].r1_ctg_ref_dict_rd2:
                    round_2_MappingRecord_dict[read_id_base].r1_ctg_ref_dict_rd2[ref_id] = {ref_pos: cigar}
                else:
                    round_2_MappingRecord_dict[read_id_base].r1_ctg_ref_dict_rd2[ref_id][ref_pos] = cigar

            if read_strand == '2':
                if ref_id not in round_2_MappingRecord_dict[read_id_base].r2_ctg_ref_dict_rd2:
                    round_2_MappingRecord_dict[read_id_base].r2_ctg_ref_dict_rd2[ref_id] = {ref_pos: cigar}
                else:
                    round_2_MappingRecord_dict[read_id_base].r2_ctg_ref_dict_rd2[ref_id][ref_pos] = cigar


####################################################################################################

n = 0
processed_num = 0
to_extract_read_base_rd2_ctg = set()
for each_mp in round_2_MappingRecord_dict.copy():

    ctg_mp_record = round_2_MappingRecord_dict[each_mp]
    ctg_refs_to_ignore_rd2 = set()

    ########## get lowest mismatch for r1/r2 ctg refs ##########

    # get r1_ref_cigar_set
    r1_ref_cigar_set = set()
    for each_pos_dict in ctg_mp_record.r1_ctg_ref_dict_rd2.values():
        each_pos_dict_values = {each_pos_dict[i] for i in each_pos_dict}
        r1_ref_cigar_set.update(each_pos_dict_values)

    # get r2_ref_cigar_set
    r2_ref_cigar_set = set()
    for each_pos_dict in ctg_mp_record.r2_ctg_ref_dict_rd2.values():
        each_pos_dict_values = {each_pos_dict[i] for i in each_pos_dict}
        r2_ref_cigar_set.update(each_pos_dict_values)

    r1_ref_min_mismatch = get_min_mismatch_from_cigar_list(r1_ref_cigar_set, min_M_len_ctg)
    r2_ref_min_mismatch = get_min_mismatch_from_cigar_list(r2_ref_cigar_set, min_M_len_ctg)
    round_2_MappingRecord_dict[each_mp].r1_ctg_refs_lowest_mismatch_rd2 = r1_ref_min_mismatch
    round_2_MappingRecord_dict[each_mp].r2_ctg_refs_lowest_mismatch_rd2 = r2_ref_min_mismatch

    ########## filter r1 16s refs ##########

    r1_ctg_refs_passed_qc = {}
    for r1_ctg_ref_rd2 in ctg_mp_record.r1_ctg_ref_dict_rd2:
        r1_matched_pos_dict = ctg_mp_record.r1_ctg_ref_dict_rd2[r1_ctg_ref_rd2]
        if len(r1_matched_pos_dict) > 1:
            ctg_refs_to_ignore_rd2.add(r1_ctg_ref_rd2)
        else:
            r1_ctg_ref_pos = list(r1_matched_pos_dict.keys())[0]
            r1_ctg_ref_cigar = r1_matched_pos_dict[r1_ctg_ref_pos]
            r1_ctg_ref_cigar_splitted = cigar_splitter(r1_ctg_ref_cigar)

            # check both end clip
            both_end_clp = check_both_ends_clipping(r1_ctg_ref_cigar_splitted)
            if both_end_clp is True:
                ctg_refs_to_ignore_rd2.add(r1_ctg_ref_rd2)
            else:
                # check mismatch
                r1_aligned_len, r1_aligned_pct, r1_clipping_len, r1_clipping_pct, r1_mismatch_pct = get_cigar_stats(r1_ctg_ref_cigar_splitted)
                if r1_ref_min_mismatch == 'NA':
                    ctg_refs_to_ignore_rd2.add(r1_ctg_ref_rd2)
                elif (r1_mismatch_pct > r1_ref_min_mismatch) or (r1_mismatch_pct > mismatch_cutoff):
                    ctg_refs_to_ignore_rd2.add(r1_ctg_ref_rd2)
                else:
                    # check aligned length
                    if r1_aligned_len < min_M_len_ctg:
                        ctg_refs_to_ignore_rd2.add(r1_ctg_ref_rd2)
                    else:
                        # check if clp in the middle
                        clip_in_middle = False
                        if ('S' in r1_ctg_ref_cigar) or ('s' in r1_ctg_ref_cigar):
                            clip_in_middle = True
                            if (r1_ctg_ref_cigar_splitted[0][-1] in ['S', 's']) and (r1_ctg_ref_pos == 1):
                                clip_in_middle = False
                            if (r1_ctg_ref_cigar_splitted[-1][-1] in ['S', 's']):
                                if (r1_ctg_ref_pos + r1_aligned_len - 1) == round_2_ctg_end_seq_len_dict[r1_ctg_ref_rd2]:
                                    clip_in_middle = False

                        if clip_in_middle is True:
                            ctg_refs_to_ignore_rd2.add(r1_ctg_ref_rd2)
                        else:
                            r1_ctg_refs_passed_qc[r1_ctg_ref_rd2] = [r1_ctg_ref_cigar]

    ########## filter r2 16s refs ##########

    r2_ctg_refs_passed_qc = {}
    for r2_ctg_ref_rd2 in ctg_mp_record.r2_ctg_ref_dict_rd2:
        r2_matched_pos_dict = ctg_mp_record.r2_ctg_ref_dict_rd2[r2_ctg_ref_rd2]
        if len(r2_matched_pos_dict) > 1:
            ctg_refs_to_ignore_rd2.add(r2_ctg_ref_rd2)
        else:
            r2_ctg_ref_pos = list(r2_matched_pos_dict.keys())[0]
            r2_ctg_ref_cigar = r2_matched_pos_dict[r2_ctg_ref_pos]
            r2_ctg_ref_cigar_splitted = cigar_splitter(r2_ctg_ref_cigar)

            # check both end clip
            both_end_clp = check_both_ends_clipping(r2_ctg_ref_cigar_splitted)
            if both_end_clp is True:
                ctg_refs_to_ignore_rd2.add(r2_ctg_ref_rd2)
            else:
                # check mismatch
                r2_aligned_len, r2_aligned_pct, r2_clipping_len, r2_clipping_pct, r2_mismatch_pct = get_cigar_stats(r2_ctg_ref_cigar_splitted)
                if r2_ref_min_mismatch == 'NA':
                    ctg_refs_to_ignore_rd2.add(r2_ctg_ref_rd2)
                elif (r2_mismatch_pct > r2_ref_min_mismatch) or (r2_mismatch_pct > mismatch_cutoff):
                    ctg_refs_to_ignore_rd2.add(r2_ctg_ref_rd2)
                else:
                    # check aligned length
                    if r2_aligned_len < min_M_len_ctg:
                        ctg_refs_to_ignore_rd2.add(r2_ctg_ref_rd2)
                    else:
                        # check if clp in the middle
                        clip_in_middle = False
                        if ('S' in r2_ctg_ref_cigar) or ('s' in r2_ctg_ref_cigar):
                            clip_in_middle = True
                            if (r2_ctg_ref_cigar_splitted[0][-1] in ['S', 's']) and (r2_ctg_ref_pos == 1):
                                clip_in_middle = False
                            if (r2_ctg_ref_cigar_splitted[-1][-1] in ['S', 's']):
                                if (r2_ctg_ref_pos + r2_aligned_len - 1) == round_2_ctg_end_seq_len_dict[r2_ctg_ref_rd2]:
                                    clip_in_middle = False

                        if clip_in_middle is True:
                            ctg_refs_to_ignore_rd2.add(r2_ctg_ref_rd2)
                        else:
                            r2_ctg_refs_passed_qc[r2_ctg_ref_rd2] = [r2_ctg_ref_cigar]

    ####################################################################################################

    r1_ctg_refs_rd2_no_ignored = {key: value for key, value in r1_ctg_refs_passed_qc.items() if key not in ctg_refs_to_ignore_rd2}
    r2_ctg_refs_rd2_no_ignored = {key: value for key, value in r2_ctg_refs_passed_qc.items() if key not in ctg_refs_to_ignore_rd2}

    # only r1 has no_ignored alignments
    if (len(r1_ctg_refs_rd2_no_ignored) > 0) and (len(r2_ctg_refs_rd2_no_ignored) == 0):
        round_2_MappingRecord_dict[each_mp].qualified_reads_rd2 = True
        to_extract_read_base_rd2_ctg.add(each_mp)

    # only r2 has no_ignored alignments
    if (len(r1_ctg_refs_rd2_no_ignored) == 0) and (len(r2_ctg_refs_rd2_no_ignored) > 0):
        round_2_MappingRecord_dict[each_mp].qualified_reads_rd2 = True
        to_extract_read_base_rd2_ctg.add(each_mp)



# extract reads with seqtk
seqtk_extract_cmd_rd1_r1 = 'seqtk subseq %s %s > %s' % (reads_file_r1_fasta, rd2_to_extract_flking_ctg_r1_id, rd2_extracted_flking_ctg_r1_seq)
seqtk_extract_cmd_rd1_r2 = 'seqtk subseq %s %s > %s' % (reads_file_r2_fasta, rd2_to_extract_flking_ctg_r2_id, rd2_extracted_flking_ctg_r2_seq)
#report_and_log((seqtk_extract_cmd_rd1_r1), pwd_log_file, True)
#report_and_log((seqtk_extract_cmd_rd1_r2), pwd_log_file, True)
#os.system(seqtk_extract_cmd_rd1_r1)
#os.system(seqtk_extract_cmd_rd1_r2)

# read extracted read sequences into dict
extract_rd2_flking_ctg_read_seq_dict = {}
for extracted_r1 in SeqIO.parse(rd2_extracted_flking_ctg_r1_seq_tmp, 'fasta'):
    extract_rd2_flking_ctg_read_seq_dict[extracted_r1.id] = str(extracted_r1.seq)
for extracted_r2 in SeqIO.parse(rd2_extracted_flking_ctg_r2_seq_tmp, 'fasta'):
    extract_rd2_flking_ctg_read_seq_dict[extracted_r2.id] = str(extracted_r2.seq)

# write out paired in the same order
rd2_extracted_flking_ctg_r1_handle = open(rd2_extracted_flking_ctg_r1_seq, 'w')
rd2_extracted_flking_ctg_r2_handle = open(rd2_extracted_flking_ctg_r2_seq, 'w')
for each_read_base in to_extract_read_base_rd2_ctg:
    current_r1 = '%s.1' % each_read_base
    current_r2 = '%s.2' % each_read_base
    current_r1_seq = extract_rd2_flking_ctg_read_seq_dict.get(current_r1, '')
    current_r2_seq = extract_rd2_flking_ctg_read_seq_dict.get(current_r2, '')
    rd2_extracted_flking_ctg_r1_handle.write('>%s\n' % current_r1)
    rd2_extracted_flking_ctg_r1_handle.write('%s\n' % current_r1_seq)
    rd2_extracted_flking_ctg_r2_handle.write('>%s\n' % current_r2)
    rd2_extracted_flking_ctg_r2_handle.write('%s\n' % current_r2_seq)
rd2_extracted_flking_ctg_r1_handle.close()
rd2_extracted_flking_ctg_r2_handle.close()





import os
import glob
from Bio import SeqIO


wd                  = '/Users/songweizhi/Desktop/CAMI_high'
gnm_folder          = '%s/ref_gnm_596'                                                  % wd
ffn_folder          = '%s/ref_gnm_596_barrnap_outputs'                                  % wd
ref_596_genomes     = '%s/goldstandard_high_pool.filtered.profile.stain.596_ref.txt'    % wd
len_cutoff_16s      = 1000
len_cutoff_ctg      = 5000


qualified_16s_seq    = '%s/ref_16S_%sbp_%sbp.ffn'         % (wd, len_cutoff_16s, len_cutoff_ctg)
qualified_gnm_folder = '%s/ref_with_16S_%sbp_%sbp'        % (wd, len_cutoff_16s, len_cutoff_ctg)


genomes_to_tax_dict = {}
for each_ref in open(ref_596_genomes):
    each_ref_split = each_ref.strip().split('\t')
    tax_id = each_ref_split[0]
    cami_genome_id = each_ref_split[5]
    genomes_to_tax_dict[cami_genome_id] = tax_id


if os.path.isdir(qualified_gnm_folder) is True:
    os.system('rm -r %s' % qualified_gnm_folder)
os.system('mkdir %s' % qualified_gnm_folder)

ffn_file_re = '%s/*.ffn' % ffn_folder
ffn_file_list = [os.path.basename(file_name) for file_name in glob.glob(ffn_file_re)]

qualified_16s_seq_handle = open(qualified_16s_seq, 'w')
qualified_gnm_set = set()
for each_ffn in ffn_file_list:

    gnm_id = '.'.join(each_ffn.split('.')[:-1])

    pwd_gnm = '%s/%s.fna'   % (gnm_folder, gnm_id)
    pwd_ffn = '%s/%s'       % (ffn_folder, each_ffn)

    seq_len_dict = {}
    for each_ctg in SeqIO.parse(pwd_gnm, 'fasta'):
        seq_len_dict[each_ctg.id] = len(str(each_ctg.seq))

    index_16s = 1
    for each_rna in SeqIO.parse(pwd_ffn, 'fasta'):
        rna_id = each_rna.id
        if '16S_rRNA' in rna_id:
            rna_id_split = rna_id.split(':')
            located_ctg = rna_id_split[2]
            rna_len = len(str(each_rna.seq))
            ctg_len = seq_len_dict[located_ctg]
            if (rna_len >= len_cutoff_16s) and (ctg_len >= len_cutoff_ctg):
                qualified_gnm_set.add(gnm_id)

                # move genome to qualified folder
                os.system('cp %s %s/' % (pwd_gnm, qualified_gnm_folder))
                qualified_16s_seq_handle.write('>%s_16S_%s\n' % (gnm_id, index_16s))
                qualified_16s_seq_handle.write('%s\n' % str(each_rna.seq))
                index_16s += 1

qualified_16s_seq_handle.close()

# write out
qualified_gnm_id_file = '%s/ref_with_16S_%sbp_%sbp_%s.txt' % (wd, len_cutoff_16s, len_cutoff_ctg, len(qualified_gnm_set))
qualified_gnm_id_file_handle = open(qualified_gnm_id_file, 'w')
#squalified_gnm_id_file_handle.write('cami_genome_id\ttax_id\n')
for qualified_gnm in sorted([i for i in qualified_gnm_set]):
    #qualified_gnm_id_file_handle.write('%s\t%s\n' % (qualified_gnm, genomes_to_tax_dict[qualified_gnm]))
    qualified_gnm_id_file_handle.write('%s\n' % qualified_gnm)
qualified_gnm_id_file_handle.close()


import os
from Bio import SeqIO

# wd = '/Users/songweizhi/Desktop/lc'
# lc_30_profile               = '/Users/songweizhi/Desktop/lc/30_profile.tsv'
# complete_fna_gi_acc_tax_tsv = '/Users/songweizhi/Desktop/lc/complete.fna.gi_acc_tax.tsv'
# bacteria_fna                = 'bacteria.fna'
# ref_gnm_folder = '%s/ref_genomes' % wd

complete_fna_gi_acc_tax_tsv         = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/refseq/complete.fna.gi_acc_tax.tsv'
bacteria_fna_gi_acc_tax_tsv         = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/genomes/bacteria/bacteria.fna.gi_acc_tax.tsv'
bacteria_draft_fna_gi_acc_tax_tsv   = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/genomes/bacteria_draft/bacteria_draft.fna.gi_acc_tax.tsv'
complete_fna                        = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/refseq/complete.fna'
bacteria_fna                        = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/genomes/bacteria/bacteria.fna'
bacteria_draft_fna                  = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/genomes/bacteria_draft/bacteria_draft.fna'


lc_30_profile                       = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/2_Medium_Complexity/225_profile_d1_180bp.tsv'
lc_30_profile                       = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/1_Low_Complexity/30_profile.tsv'
lc_30_profile                       = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/450_profile_d1.tsv'
ref_gnm_folder                      = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/ref_genomes_hc_04_20'

'''

grep 'NZ_KB912814.1' /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/genomes/bacteria/bacteria.fna.id.lines.txt
grep 'NZ_KB912814.1' /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/genomes/bacteria_draft/bacteria_draft.fna.id.lines.txt
grep 'NZ_KB912814.1' /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/PROCESSED_NCBI/refseq/complete.fna.id.txt

'''


missing_tax_lc = ['1131272.1', '1230476.1']

os.mkdir(ref_gnm_folder)
if os.path.isdir(ref_gnm_folder) is True:
    os.system('rm -r %s' % ref_gnm_folder)
os.mkdir(ref_gnm_folder)

select_ref_tax_id_set = set()
for each_line in open(lc_30_profile):
    each_line_split = each_line.strip().split('\t')
    if len(each_line_split) > 1:
        if each_line_split[1] == 'strain':
                select_ref_tax_id_set.add(each_line_split[0])

select_ref_tax_id_set_no_version = {i.split('.')[0] for i in select_ref_tax_id_set}


#################### get gi_to_tax_dict ####################

gi_to_tax_dict = {}

for each_line in open(complete_fna_gi_acc_tax_tsv):
    each_line_split = each_line.strip().split('\t')
    id_gi  = each_line_split[0]
    id_acc = each_line_split[1]
    id_tax = each_line_split[2]
    if id_tax in select_ref_tax_id_set_no_version:
        gi_to_tax_dict[id_gi] = id_tax

for each_line in open(bacteria_fna_gi_acc_tax_tsv):
    each_line_split = each_line.strip().split('\t')
    id_gi  = each_line_split[0]
    id_acc = each_line_split[1]
    id_tax = each_line_split[2]
    if id_tax in select_ref_tax_id_set_no_version:
        gi_to_tax_dict[id_gi] = id_tax

for each_line in open(bacteria_draft_fna_gi_acc_tax_tsv):
    each_line_split = each_line.strip().split('\t')
    id_gi  = each_line_split[0]
    id_acc = each_line_split[1]
    id_tax = each_line_split[2]
    if id_tax in select_ref_tax_id_set_no_version:
        gi_to_tax_dict[id_gi] = id_tax


############################################################

for each_seq in SeqIO.parse(bacteria_fna, 'fasta'):
    gi_id = each_seq.id.split('|')[1]
    if gi_id in gi_to_tax_dict:
        tax_id = gi_to_tax_dict[gi_id]
        pwd_gnm_file = '%s/tax_%s.fa' % (ref_gnm_folder, tax_id)
        with open(pwd_gnm_file, 'a') as gnm_file_handle:
            gnm_file_handle.write('>%s\n' % each_seq.id)
            gnm_file_handle.write('%s\n' % str(each_seq.seq))

for each_seq in SeqIO.parse(bacteria_draft_fna, 'fasta'):
    gi_id = each_seq.id.split('|')[1]
    if gi_id in gi_to_tax_dict:
        tax_id = gi_to_tax_dict[gi_id]
        pwd_gnm_file = '%s/tax_%s.fa' % (ref_gnm_folder, tax_id)
        with open(pwd_gnm_file, 'a') as gnm_file_handle:
            gnm_file_handle.write('>%s\n' % each_seq.id)
            gnm_file_handle.write('%s\n' % str(each_seq.seq))

for each_seq in SeqIO.parse(complete_fna, 'fasta'):
    gi_id = each_seq.id.split('|')[1]
    if gi_id in gi_to_tax_dict:
        tax_id = gi_to_tax_dict[gi_id]
        pwd_gnm_file = '%s/tax_%s.fa' % (ref_gnm_folder, tax_id)
        with open(pwd_gnm_file, 'a') as gnm_file_handle:
            gnm_file_handle.write('>%s\n' % each_seq.id)
            gnm_file_handle.write('%s\n' % str(each_seq.seq))




import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


value_list_x    = [1,2,3,4,5,3,2,5]
value_list_y    = [1,2,3,4,5,3,2,5]
pwd_figure      = '/Users/songweizhi/Desktop/multi_subplots.png'
axis_label_x    = 'Number of 16S rRNA gene in genome'
axis_label_y    = 'Linked 16S rRNA gene'


plt.scatter(value_list_x, value_list_y, c="b")
plt.xlabel(axis_label_x)
plt.ylabel(axis_label_y)
plt.tight_layout()
plt.savefig(pwd_figure)
plt.close()





def subset_prokaryotes_csv(GTDB_genomes, GenBank_genomes, GenBank_genomes_subset):

    num_to_full_id_dict = {}
    GTDB_genome_id_list = set()
    for each in open(GTDB_genomes):
        if not each.startswith('"ID",'):
            each_split = each.strip().split(',')
            NCBI_ID_with_version = each_split[0][1:-1]
            NCBI_ID_without_version = '.'.join(NCBI_ID_with_version.split('.')[:-1])
            NCBI_ID_no_version_only_number = NCBI_ID_without_version[3:]
            GTDB_genome_id_list.add(NCBI_ID_no_version_only_number)
            num_to_full_id_dict[NCBI_ID_no_version_only_number] = NCBI_ID_with_version

    GenBank_genomes_subset_handle = open(GenBank_genomes_subset, 'w')
    found_in_GenBank_genome_id_list = set()
    for GenBank_genome in open(GenBank_genomes):
        if not GenBank_genome.startswith('#Organism Name'):
            GenBank_genome_split = GenBank_genome.strip().split(',')
            GenBank_genome_id_with_version = GenBank_genome_split[5][1:-1]
            GenBank_genome_id_without_version = '.'.join(GenBank_genome_id_with_version.split('.')[:-1])
            GenBank_genome_id_no_version_only_number = GenBank_genome_id_without_version[3:]
            if GenBank_genome_id_no_version_only_number in GTDB_genome_id_list:
                found_in_GenBank_genome_id_list.add(GenBank_genome_id_no_version_only_number)
                GenBank_genomes_subset_handle.write(GenBank_genome)
    GenBank_genomes_subset_handle.close()

    for each in GTDB_genome_id_list:
        if each not in found_in_GenBank_genome_id_list:
            print('not found: %s' % num_to_full_id_dict[each])


def subset_prokaryotes_csv_by_accession(GenBank_genomes, accession_file, GenBank_genomes_subset):

    num_to_full_id_dict = {}
    genomes_in_accession = set()
    for each in open(accession_file):
        if not each.startswith('"ID",'):
            each_split = each.strip().split(',')
            NCBI_ID_with_version = each_split[0][1:-1]
            NCBI_ID_without_version = '.'.join(NCBI_ID_with_version.split('.')[:-1])
            NCBI_ID_no_version_only_number = NCBI_ID_without_version[3:]
            genomes_in_accession.add(NCBI_ID_no_version_only_number)
            num_to_full_id_dict[NCBI_ID_no_version_only_number] = NCBI_ID_with_version

    GenBank_genomes_subset_handle = open(GenBank_genomes_subset, 'w')
    found_in_GenBank_genome_id_list = set()
    for GenBank_genome in open(GenBank_genomes):
        if not GenBank_genome.startswith('#Organism Name'):
            GenBank_genome_split = GenBank_genome.strip().split(',')
            GenBank_genome_id_with_version = GenBank_genome_split[5][1:-1]
            GenBank_genome_id_without_version = '.'.join(GenBank_genome_id_with_version.split('.')[:-1])
            GenBank_genome_id_no_version_only_number = GenBank_genome_id_without_version[3:]
            if GenBank_genome_id_no_version_only_number in genomes_in_accession:
                found_in_GenBank_genome_id_list.add(GenBank_genome_id_no_version_only_number)
                GenBank_genomes_subset_handle.write(GenBank_genome)
    GenBank_genomes_subset_handle.close()

    for each in genomes_in_accession:
        if each not in found_in_GenBank_genome_id_list:
            print('not found: %s' % num_to_full_id_dict[each])

GenBank_genomes                                 = '/Users/songweizhi/Desktop/select_strain/prokaryotes.csv'


GTDB_Pseudomonas_csv                            = '/Users/songweizhi/Desktop/select_strain/GTDB-Pseudomonas.csv'
GTDB_o__Pseudomonadales_csv                     = '/Users/songweizhi/Desktop/select_strain/GTDB-o__Pseudomonadales.csv'
GTDB_o__Pseudomonadales_no_Pseudomonadaceae_csv = '/Users/songweizhi/Desktop/select_strain/GTDB-o__Pseudomonadales_no_Pseudomonadaceae.csv'
GTDB_f__Pseudomonadaceae_no_Pseudomonas_E_csv   = '/Users/songweizhi/Desktop/select_strain/GTDB-f__Pseudomonadaceae_no_Pseudomonas_E.csv'
GTDB_f__Pseudomonadaceae_csv                    = '/Users/songweizhi/Desktop/select_strain/GTDB-f__Pseudomonadaceae.csv'
GTDB_g__Pseudomonas_E_csv                       = '/Users/songweizhi/Desktop/select_strain/GTDB-Pseudomonas_E.csv'
GenBank_genomes_subset_g__Pseudomonas_E         = '/Users/songweizhi/Desktop/select_strain/GenBank_g__Pseudomonas_E_subset.csv'
GenBank_genomes_subset_f__Pseudomonadaceae_no_Pseudomonas_E   = '/Users/songweizhi/Desktop/select_strain/GenBank_f__Pseudomonadaceae_subset_no_Pseudomonas_E.csv'
GenBank_genomes_subset_o__Pseudomonadales_no_Pseudomonadaceae = '/Users/songweizhi/Desktop/select_strain/GenBank_o__Pseudomonadales_no_Pseudomonadaceae.csv'


GTDB_Pseudomonas_E_csv_handle = open(GTDB_g__Pseudomonas_E_csv, 'w')
for genome in open(GTDB_Pseudomonas_csv):
    genome_split = genome.strip().split(',')
    GTDB_Taxonomy = genome_split[3]
    if 'g__Pseudomonas_E' in GTDB_Taxonomy:
        GTDB_Pseudomonas_E_csv_handle.write(genome)
GTDB_Pseudomonas_E_csv_handle.close()


subset_prokaryotes_csv(GTDB_g__Pseudomonas_E_csv, GenBank_genomes, GenBank_genomes_subset_g__Pseudomonas_E)
subset_prokaryotes_csv(GTDB_f__Pseudomonadaceae_no_Pseudomonas_E_csv, GenBank_genomes, GenBank_genomes_subset_f__Pseudomonadaceae_no_Pseudomonas_E)
subset_prokaryotes_csv(GTDB_o__Pseudomonadales_no_Pseudomonadaceae_csv, GenBank_genomes, GenBank_genomes_subset_o__Pseudomonadales_no_Pseudomonadaceae)


# for line in open(GenBank_genomes_subset_g__Pseudomonas_E):
#     line_split = line.strip().split(',')
#     if line_split[6] in ['"Complete"', '" Chromosome"']:
#         pass
#         #print(line.strip())
#         #print()


# for line_f in open(GTDB_o__Pseudomonadales_csv):
#     line_f_split = line_f.strip().split(',')
#     GTDB_Taxonomy = line_f_split[3]
#
#     if 'f__Pseudomonadaceae' not in GTDB_Taxonomy:
#         print(line_f.strip())
#



# for line in open(GenBank_genomes_subset_o__Pseudomonadales_no_Pseudomonadaceae):
#     line_split = line.strip().split(',')
#     if line_split[6] in ['"Complete"', '" Chromosome"']:
#
#         print(line.strip())
#         #print()




'''

cd /Users/songweizhi/Desktop/select_strain
BioSAK dwnld_GenBank_genome -csv GenBank_Pseudomonas_E_subset.csv -fna -name -t 4
BioSAK dwnld_GenBank_genome -csv GenBank_Pseudomonas_E_subset_completed.csv -fna -name -t 4

BioSAK dwnld_GenBank_genome -csv GenBank_f__Pseudomonadaceae_subset_no_Pseudomonas_E_completed.csv -fna -name -t 4


'''

x= 0
for line3 in open('/Users/songweizhi/Desktop/select_strain/bac120_metadata_r95.tsv'):
    line3_split = line3.strip().split('\t')
    accession = line3_split[0]
    gtdb_taxonomy = line3_split[16]
    ncbi_assembly_level = line3_split[45]
    ncbi_type_material_designation = line3_split[84]
    ssu_count = line3_split[91]

    if ('d__Bacteria' in gtdb_taxonomy) and (ncbi_assembly_level not in ['Contig', 'Scaffold']) and ('p__Proteobacteria' not in gtdb_taxonomy) and (ncbi_type_material_designation == 'assembly from type material') and (line3_split[17] == 'type strain of species'):
            #and ('g__Pseudomonas_E' not in gtdb_taxonomy) and (ncbi_assembly_level not in ['Contig', 'Scaffold']) and ('g__Pseudomonas_D' in gtdb_taxonomy):
            #
            #


        print('%s\t%s' % (accession, gtdb_taxonomy))
        #print(line3_split[17])
        #print(ncbi_type_material_designation)
        x += 1
print(x)


import os
from Bio import SeqIO


wd                  = '/Users/songweizhi/Desktop/CAMI_1'
combined_gnm_file   = '%s/S_S001__genomes_30__insert_180_gsa_anonymous.fasta'   % wd
ctg_to_gnm_file     = '%s/S_S001__genomes_30__insert_180_gsa_mapping.tsv'       % wd

ref_gnm_folder = '%s/ref_genomes' % wd


os.mkdir(ref_gnm_folder)
if os.path.isdir(ref_gnm_folder) is True:
    os.system('rm -r %s' % ref_gnm_folder)
os.mkdir(ref_gnm_folder)


ctg_to_gnm_dict = {}
ctg_to_seq_id = {}
for each_ctg in open(ctg_to_gnm_file):
    each_ctg_split = each_ctg.strip().split('\t')
    ctg_id = each_ctg_split[0]
    gnm_id = each_ctg_split[1]
    ctg_id_raw = each_ctg_split[3]
    ctg_to_seq_id[ctg_id] = ctg_id_raw
    ctg_to_gnm_dict[ctg_id] = gnm_id


for each_seq in SeqIO.parse(combined_gnm_file, 'fasta'):

    seq_id = each_seq.id
    seq_id_raw = ctg_to_seq_id[seq_id]
    seq_gnm = ctg_to_gnm_dict[seq_id]

    pwd_gnm_file = '%s/%s.fa' % (ref_gnm_folder, seq_gnm)
    print(pwd_gnm_file)
    with open(pwd_gnm_file, 'a') as gnm_file_handle:
        gnm_file_handle.write('>%s\n' % seq_id_raw)
        gnm_file_handle.write('%s\n' % str(each_seq.seq))







def check_cigar_quality_consider_ref_min_mismatch(cigar_str, ref_min_mismatch, mismatch_cutoff, min_M_len, ref_pos, marker_len):

    cigar_splitted = cigar_splitter(cigar_str)
    qualified_cigar = False

    # check both end clip
    both_end_clp = check_both_ends_clipping(cigar_splitted)
    if both_end_clp is False:
        # check mismatch
        aln_len, aln_pct, clp_len, clp_pct, mismatch_pct = get_cigar_stats(cigar_splitted)

        if ref_min_mismatch != 'NA':
            if (mismatch_pct <= ref_min_mismatch) and (mismatch_pct <= mismatch_cutoff):
                # check aligned length
                if aln_len >= min_M_len:
                    # check if clp in the middle
                    clip_in_middle = False
                    if ('S' in cigar_str) or ('s' in cigar_str):
                        clip_in_middle = True
                        if (cigar_splitted[0][-1] in ['S', 's']) and (ref_pos == 1):
                            clip_in_middle = False
                        if (cigar_splitted[-1][-1] in ['S', 's']):
                            if (ref_pos + aln_len - 1) == marker_len:
                                clip_in_middle = False

                    if clip_in_middle is False:
                        qualified_cigar = True

    return qualified_cigar




import os

js_header = '''#!/bin/bash
#PBS -l nodes=1:ppn=16
#PBS -l mem=90gb
#PBS -l walltime=71:59:00
#PBS -j oe
#PBS -M weizhi.song@unsw.edu.au
#PBS -m ae

module load python/3.6.5
source ~/mypython365env/bin/activate
module load java/7u51
module load gcc/7.3.0
module load sparsehash/2.0.3
module load matam/1.5.3
module load samtools/1.9
module load usearch/11.0.667
module load seqtk/20190219
export PATH=/home/z5039045/anaconda3/bin:$PATH

cd /srv/scratch/z5039045/Kelp_16S
'''

Kelp_sample_id = '/Users/songweizhi/Desktop/Kelp_sample_id.txt'
js_folder = '/Users/songweizhi/Desktop/js_matam_16s'


co_assembly_dict = {}
for co_assembly in open(Kelp_sample_id):
    co_assembly_split = co_assembly.strip().split('\t')
    if co_assembly_split[0] not in co_assembly_dict:
        co_assembly_dict[co_assembly_split[0]] = [co_assembly_split[1]]
    else:
        co_assembly_dict[co_assembly_split[0]].append(co_assembly_split[1])


for each_co_assembly in co_assembly_dict:
    #print(each_co_assembly)

    js_file = '%s/js_matam_%s.sh' % (js_folder, each_co_assembly)
    js_file_handle = open(js_file, 'w')
    js_file_handle.write(js_header)
    js_file_handle.write('MarkerMAG matam_16s -p %s -r1 %s_R1.fasta -r2 %s_R2.fasta -pct 0.1,0.5,1,5,10,25,50,75 -ref /srv/scratch/z5039045/DB/Matam/SILVA_128_SSURef_NR95 -i 0.995 -t 16 -force -matam_assembly /home/z5039045/anaconda3/pkgs/matam-v1.5.3-0/bin/matam_assembly.py -sortmerna /home/z5039045/anaconda3/pkgs/matam-v1.5.3-0/opt/matam-v1.5.3/sortmerna/sortmerna\n' % (each_co_assembly, each_co_assembly, each_co_assembly))
    js_file_handle.close()





js_header = '''#!/bin/bash
#PBS -l nodes=1:ppn=12
#PBS -l mem=30gb
#PBS -l walltime=11:59:00
#PBS -j oe
#PBS -M weizhi.song@unsw.edu.au
#PBS -m ae

module load samtools/1.10
module load bowtie/2.3.5.1
cd /srv/scratch/z5039045/MarkerMAG_wd/Tara_SOC/2_mapping
'''

js_folder = '/Users/songweizhi/Desktop/Tara_SOC_js_mapping'


for each_sample in ['ERR598945', 'ERR599059', 'ERR3587138', 'ERR3587191', 'ERR599090', 'ERR599176', 'ERR3587139', 'ERR3587192', 'ERR599104', 'ERR599121', 'ERR3587140', 'ERR3587193']:

    pwd_js = '%s/js_%s.sh' % (js_folder, each_sample)

    r1_qc_p  = '%s_R1.fasta'   % each_sample
    r2_qc_p  = '%s_R2.fasta'   % each_sample
    cmd_mapping = 'bowtie2 -x TARA_SOC_RAW_min2500 -1 ../1_raw_reads/%s -2 ../1_raw_reads/%s -S %s.sam -f -p 12\n' % (r1_qc_p, r2_qc_p, each_sample)
    cmd_sam2bam = 'samtools view -bS %s.sam -o %s.bam\n' % (each_sample, each_sample)
    cmd_sort    = 'samtools sort %s.bam -o %s_sorted.bam\n' % (each_sample, each_sample)
    cmd_index   = 'samtools index %s_sorted.bam\n' % each_sample
    cmd_rm_sam  = 'rm %s.sam\n' % each_sample
    cmd_rm_bam  = 'rm %s.bam\n' % each_sample

    pwd_js_handle = open(pwd_js, 'w')
    pwd_js_handle.write(js_header)
    pwd_js_handle.write(cmd_mapping)
    pwd_js_handle.write(cmd_sam2bam)
    pwd_js_handle.write(cmd_sort)
    pwd_js_handle.write(cmd_index)
    pwd_js_handle.write(cmd_rm_sam)
    pwd_js_handle.write(cmd_rm_bam)
    pwd_js_handle.close()



from Bio import SeqIO


def get_regions_to_ignore_from_barrnap_output(combined_barrnap_gff, ctg_len_dict):

    ctg_ignore_region_dict = {}

    for each_line in open(combined_barrnap_gff):
        if (not each_line.startswith('#')) and ('16S_rRNA' in each_line):
            each_line_split = each_line.strip().split('\t')
            ctg_id = each_line_split[0]
            ctg_len = ctg_len_dict[ctg_id]
            start_pos = int(each_line_split[3])
            end_pos = int(each_line_split[4])
            len_16s = end_pos - start_pos + 1
            left_gap = start_pos - 1
            right_gap = ctg_len - end_pos - 1

            # print(each_line_split)
            print('%s(%sbp)\t%s\t%s\t%s' % (ctg_id, ctg_len, len_16s, left_gap, right_gap))
            # print()

            if left_gap <= 50:
                if ctg_id not in ctg_ignore_region_dict:
                    ctg_ignore_region_dict[ctg_id] = {'left_end'}
                else:
                    ctg_ignore_region_dict[ctg_id].add('left_end')

            if right_gap <= 50:
                if ctg_id not in ctg_ignore_region_dict:
                    ctg_ignore_region_dict[ctg_id] = {'right_end'}
                else:
                    ctg_ignore_region_dict[ctg_id].add('right_end')

    return ctg_ignore_region_dict


combined_gnm_file       = '/Users/songweizhi/Desktop/GI_refined_bins_combined.fa'
combined_barrnap_gff    = '/Users/songweizhi/Desktop/combined.gff'


ctg_len_dict = {}
for each_ctg in SeqIO.parse(combined_gnm_file, 'fasta'):
    ctg_len_dict[each_ctg.id] = len(each_ctg.seq)


ctg_ignore_region_dict = get_regions_to_ignore_from_barrnap_output(combined_barrnap_gff, ctg_len_dict)
print(ctg_ignore_region_dict)






from Bio import SeqIO


fasta_in  = '/Users/songweizhi/Desktop/CB_ER_070716.fasta'
fasta_out = '/Users/songweizhi/Desktop/CB_ER_070716_renamed.fasta'

fasta_in  = 'combined_16s_reads.fa'
fasta_out = 'combined_16s_reads_renamed.fa'


fasta_out_handle = open(fasta_out, 'w')
for read_record in SeqIO.parse(fasta_in, 'fasta'):
    read_record_description = read_record.description
    read_record_description_split = read_record_description.split(' ')
    read_record_new_id = ''
    if read_record_description_split[1][0] == '1':
        read_record_new_id = '%s.1' % read_record.id
    if read_record_description_split[1][0] == '2':
        read_record_new_id = '%s.2' % read_record.id
    fasta_out_handle.write('>%s\n' % read_record_new_id)
    fasta_out_handle.write('%s\n' % read_record.seq)
fasta_out_handle.close()



import os


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def overlap_between_list(list_1, list_2):

    overlap = False
    for list_1_element in list_1:
        if list_1_element in list_2:
            overlap = True

    return overlap


def gnm_level_asessment(linked_mag_dict_rd1):

    rd1_correct_num = 0
    rd1_wrong_gnm = set()
    rd1_unknown_gnm = set()
    rd1_ambiguous_gnm = set()
    for each_rd_1_mag in linked_mag_dict_rd1:
        mag_assess = linked_mag_dict_rd1[each_rd_1_mag]
        if len(mag_assess) == 1:
            if mag_assess == {'Correct'}:
                rd1_correct_num += 1
            elif mag_assess == {'Wrong'}:
                rd1_wrong_gnm.add(each_rd_1_mag)
            elif mag_assess == {'Unknown'}:
                rd1_unknown_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 2:
            if ('Correct' in mag_assess) and ('Unknown' in mag_assess):
                rd1_correct_num += 1
            if ('Correct' in mag_assess) and ('Wrong' in mag_assess):
                rd1_ambiguous_gnm.add(each_rd_1_mag)
            if ('Wrong' in mag_assess) and ('Unknown' in mag_assess):
                rd1_wrong_gnm.add(each_rd_1_mag)
        elif len(mag_assess) == 3:
            rd1_ambiguous_gnm.add(each_rd_1_mag)

    return rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm


########################################################################################################################

wd = '/Users/songweizhi/Desktop/assess_linkages_hc'

########## reference to cluster ##########

drep_ani_cutoff             = 97
drep_cdb_file               = '%s/file_in/Cdb_%s.csv'                                       % (wd, drep_ani_cutoff)
#ref_to_strain_file          = '%s/file_in/ref_to_strain.txt'                                % wd

########## bin to reference ##########

parse_blastn_bin_vs_ref     = False  # True or False
blastn_bin_vs_ref           = '%s/file_in/cami_hc_bin_vs_ref.tab'                                   % wd
iden_cutoff                 = 99.5
aln_len_cutoff              = 1500
cov_q_cutoff                = 90
min_match_length            = 102400  # 100Kbp:102400, 500Kbp:524288,
#mag_metadata                = '%s/file_in/MAG_metadata.txt'                                 % wd

########## 16S to reference ##########

perform_blastn_16s_vs_refs  = False  # True or False
combined_GI_ref_16S         = '%s/file_in/combined_hc_ref_16S.ffn'                                  % wd
#matam_16s_seqs              = '%s/file_in/cami_hc_SILVA138_id99_assembled_16S_uclust_0.999.fasta'   % wd
matam_16s_seqs              = '%s/file_in/cami_hc_SILVA138_uclust_0.999_polished.fa'                % wd
matam_16s_blastn            = '%s/file_in/matam_16S_uclust_0.999_vs_ref.tab'                        % wd
iden_cutoff_16s             = 99.5  # 99.3 (best), 99.5
aln_len_cutoff_16s          = 500
cov_q_cutoff_16s            = 70
total_query_mag_num         = 97

########## assessment results ##########

MarkerMAG_linkages          = '%s/hc_0519_linkages_by_genome.txt'                  % wd
#MarkerMAG_linkages          = '%s/hc_0514_stats_combined_filtered.txt'                  % wd
linkages_from_rd1           = False


########## script ##########

pwd_plot_sankey_R = '%s/file_in/get_sankey_plot.R' % wd

############################################### define file/folder name ################################################

# bin to reference
bin_vs_ref_txt              = '%s/bin_vs_ref_imag%s.txt'                        % (wd, iden_cutoff)
stats_bin_to_ref_txt        = '%s/stats_bin_to_ref_imag%s.txt'                  % (wd, iden_cutoff)
stats_ref_to_bin_txt        = '%s/stats_ref_to_bin_imag%s.txt'                  % (wd, iden_cutoff)
stats_bin_to_cluster_txt    = '%s/stats_bin_to_cluster_ani%s_imag%s.txt'        % (wd, drep_ani_cutoff, iden_cutoff)
stats_cluster_to_bin_txt    = '%s/stats_cluster_to_bin_ani%s_imag%s.txt'        % (wd, drep_ani_cutoff, iden_cutoff)

# assessment results
linkage_file_path, linkage_file_basename, linkage_file_extension = sep_path_basename_ext(MarkerMAG_linkages)
MarkerMAG_linkages_assessed = '%s/%s_with_assessment_ani%s_imag%s_i16S%s.txt'   % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
wrong_linkages_txt          = '%s/%s_wrong_ani%s_imag%s_i16S%s.txt'             % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)
unknown_linkages_txt        = '%s/%s_unknown_ani%s_imag%s_i16S%s.txt'           % (linkage_file_path, linkage_file_basename, drep_ani_cutoff, iden_cutoff, iden_cutoff_16s)


################################################# reference to cluster #################################################

cluster_to_ref_dict = {}
ref_to_cluster_dict = {}
for each_ref in open(drep_cdb_file):
    if not each_ref.startswith('genome,secondary_cluster'):
        each_ref_split = each_ref.strip().split(',')
        ref_file_name = each_ref_split[0]
        ref_file_name_no_ext = '.'.join(ref_file_name.split('.')[:-1])
        ref_cluster = 'C' + each_ref_split[1]
        ref_to_cluster_dict[ref_file_name_no_ext] = ref_cluster
        if ref_cluster not in cluster_to_ref_dict:
            cluster_to_ref_dict[ref_cluster] = [ref_file_name_no_ext]
        else:
            cluster_to_ref_dict[ref_cluster].append(ref_file_name_no_ext)


################################################### bin to reference ###################################################

bin_ref_connector           = '__|__'

# get ref_to_strain_dict
ref_to_strain_dict = {}
# for ref in open(ref_to_strain_file):
#     ref_split = ref.strip().split('\t')
#     ref_to_strain_dict[ref_split[0]] = ref_split[1]

if parse_blastn_bin_vs_ref is True:
    bin_vs_ref_dict = {}
    for match in open(blastn_bin_vs_ref):
        match_split = match.strip().split('\t')
        query = match_split[0]
        query_genome = '_'.join(query.split('_')[:3])
        subject = match_split[1]
        subject_genome = '_'.join(subject.split('_')[:2])
        iden = float(match_split[2])
        aln_len = int(match_split[3])
        query_len = int(match_split[12])
        subject_len = int(match_split[13])
        coverage_q = float(aln_len) * 100 / float(query_len)
        if (iden >= iden_cutoff) and (aln_len > aln_len_cutoff) and (coverage_q >= cov_q_cutoff):
            key_bin_ref = '%s%s%s' % (query_genome, bin_ref_connector, subject_genome)
            if key_bin_ref not in bin_vs_ref_dict:
                bin_vs_ref_dict[key_bin_ref] = aln_len
            else:
                bin_vs_ref_dict[key_bin_ref] += aln_len

    bin_vs_ref_dict_filtered = {}
    for each_key in bin_vs_ref_dict:
        if bin_vs_ref_dict[each_key] >= min_match_length:
            bin_vs_ref_dict_filtered[each_key] = bin_vs_ref_dict[each_key]

    # write out linkages
    linkage_txt_handle = open(bin_vs_ref_txt, 'w')
    linkage_txt_handle.write('Bin,Ref,Length\n')
    bin_to_ref_dict = {}
    ref_to_bin_dict = {}
    bin_to_cluster_dict = {}
    cluster_to_bin_dict = {}
    for each_linkage in bin_vs_ref_dict_filtered:
        linkage_split = each_linkage.split(bin_ref_connector)
        bin_id = linkage_split[0]
        ref_id = linkage_split[1]
        ref_cluster = ref_to_cluster_dict[ref_id]

        if bin_id not in bin_to_ref_dict:
            bin_to_ref_dict[bin_id] = {ref_id}
        else:
            bin_to_ref_dict[bin_id].add(ref_id)

        if bin_id not in bin_to_cluster_dict:
            bin_to_cluster_dict[bin_id] = {ref_cluster}
        else:
            bin_to_cluster_dict[bin_id].add(ref_cluster)

        if ref_id not in ref_to_bin_dict:
            ref_to_bin_dict[ref_id] = {bin_id}
        else:
            ref_to_bin_dict[ref_id].add(bin_id)

        if ref_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[ref_cluster] = {bin_id}
        else:
            cluster_to_bin_dict[ref_cluster].add(bin_id)

        linkage_txt_handle.write('%s,%s,%s\n' % (bin_id, ref_id, bin_vs_ref_dict_filtered[each_linkage]))
    linkage_txt_handle.close()

    # visualize
    cmd_sankey_bin_to_ref = 'Rscript %s -f %s -x %s -y %s' % (pwd_plot_sankey_R, bin_vs_ref_txt, 600, 1800)
    os.system(cmd_sankey_bin_to_ref)

    bin_to_ref_txt_handle = open(stats_bin_to_ref_txt, 'w')
    for each_bin in bin_to_ref_dict:
        bin_to_ref_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_ref_dict[each_bin])))
    bin_to_ref_txt_handle.close()

    ref_to_bin_txt_handle = open(stats_ref_to_bin_txt, 'w')
    for each_ref in ref_to_bin_dict:
        ref_to_bin_txt_handle.write('%s\t%s\t%s\n' % (each_ref, ','.join(ref_to_bin_dict[each_ref]), ref_to_strain_dict.get(each_ref, 'NA')))
    ref_to_bin_txt_handle.close()

    stats_bin_to_cluster_txt_handle = open(stats_bin_to_cluster_txt, 'w')
    for each_bin in bin_to_cluster_dict:
        stats_bin_to_cluster_txt_handle.write('%s\t%s\n' % (each_bin, ','.join(bin_to_cluster_dict[each_bin])))
    stats_bin_to_cluster_txt_handle.close()

    stats_cluster_to_bin_txt_handle = open(stats_cluster_to_bin_txt, 'w')
    for each_ref in cluster_to_bin_dict:
        stats_cluster_to_bin_txt_handle.write('%s\t%s\n' % (each_ref, ','.join(cluster_to_bin_dict[each_ref])))
    stats_cluster_to_bin_txt_handle.close()

bin_to_cluster_dict = {}
for each_match in open(stats_bin_to_cluster_txt):
    each_match_split = each_match.strip().split('\t')
    bin_to_cluster_dict[each_match_split[0]] = {i for i in each_match_split[1].split(',')}


cluster_to_bin_dict = {}
for each_bin in bin_to_cluster_dict:
    matched_clusters =  bin_to_cluster_dict[each_bin]
    for matched_cluster in matched_clusters:
        if matched_cluster not in cluster_to_bin_dict:
            cluster_to_bin_dict[matched_cluster] = {each_bin}
        else:
            cluster_to_bin_dict[matched_cluster].add(each_bin)


################################################### 16S to reference ###################################################

blast_parameters = '-evalue 1e-5 -outfmt "6 qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore qlen slen" -task blastn -num_threads 1'
blast_cmd = 'blastn -query %s -subject %s -out %s %s' % (matam_16s_seqs, combined_GI_ref_16S, matam_16s_blastn, blast_parameters)
if perform_blastn_16s_vs_refs is True:
    os.system(blast_cmd)

# get matam_16s_to_cluster_dict
matam_16s_to_cluster_dict = {}
for match in open(matam_16s_blastn):
    match_split = match.strip().split('\t')
    query = match_split[0]
    subject = match_split[1]
    subject_gnm = subject.split('_16S_')[0]
    subject_cluster = ref_to_cluster_dict[subject_gnm]
    iden = float(match_split[2])
    aln_len = int(match_split[3])
    query_len = int(match_split[12])
    subject_len = int(match_split[13])
    coverage_q = float(aln_len) * 100 / float(query_len)
    if (iden >= iden_cutoff_16s) and (aln_len > aln_len_cutoff_16s) and (coverage_q >= cov_q_cutoff_16s):
        if query not in matam_16s_to_cluster_dict:
            matam_16s_to_cluster_dict[query] = {subject_cluster}
        else:
            matam_16s_to_cluster_dict[query].add(subject_cluster)

cluster_to_matam_16s_dict = {}
for matam_16s in matam_16s_to_cluster_dict:
    matched_clusters =  matam_16s_to_cluster_dict[matam_16s]
    for matched_cluster in matched_clusters:
        if matched_cluster not in cluster_to_matam_16s_dict:
            cluster_to_matam_16s_dict[matched_cluster] = {matam_16s}
        else:
            cluster_to_matam_16s_dict[matched_cluster].add(matam_16s)


###################################################### assessment ######################################################

MarkerMAG_linkages_assessed_handle = open(MarkerMAG_linkages_assessed, 'w')
wrong_linkages_txt_handle = open(wrong_linkages_txt, 'w')
unknown_linkages_txt_handle = open(unknown_linkages_txt, 'w')
linkage_num_right = 0
linkage_num_wrong = 0
linkage_num_unknown = 0
linkage_assessment_dict = {}
for each_linkage in open(MarkerMAG_linkages):
    #if each_linkage.startswith('MarkerGene\tGenomicSeq\tLinkage\tStep'):
    if ('MarkerGene\tGenomicSeq\tLinkage\tRound' in each_linkage) or ('MarkerGene,GenomicSeq,Number' in each_linkage):
        MarkerMAG_linkages_assessed_handle.write('MarkerGene\tGenomicSeq\tLinkage\tRound\tAssessment\n')
    else:
        id_16s = ''
        id_mag = ''
        link_num = ''
        if linkages_from_rd1 is False:
            each_linkage_split = each_linkage.strip().split('\t')
            id_16s = each_linkage_split[0]
            id_mag = each_linkage_split[1]
            link_num = each_linkage_split[2]
        else:
            each_linkage_split = each_linkage.strip().split(',')
            id_16s = each_linkage_split[0][12:]
            id_mag = each_linkage_split[1][12:]
            link_num = each_linkage_split[2]

        #print('id_mag: %s' % id_mag)
        key_16s_mag = '%s___%s' % (id_16s, id_mag)

        matched_cluster_16s = {}
        if id_16s in matam_16s_to_cluster_dict:
            matched_cluster_16s = matam_16s_to_cluster_dict[id_16s]

        matched_cluster_mag = {}
        if id_mag in bin_to_cluster_dict:
            matched_cluster_mag = bin_to_cluster_dict[id_mag]

        if (matched_cluster_16s != {}) and (matched_cluster_mag != {}):

            if overlap_between_list(matched_cluster_mag, matched_cluster_16s) is True:
                linkage_num_right += 1

                if linkages_from_rd1 is False:
                    MarkerMAG_linkages_assessed_handle.write('%s\tCorrect\n' % each_linkage.strip())
                else:
                    MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tS1\tCorrect\n' % (id_16s, id_mag, link_num))

                if id_mag not in linkage_assessment_dict:
                    linkage_assessment_dict[id_mag] = ['Correct']
                else:
                    linkage_assessment_dict[id_mag].append('Correct')

            else:
                #print('matched_cluster_16s: %s' % matched_cluster_16s)
                #print('matched_cluster_mag: %s' % matched_cluster_mag)
                #print()
                linkage_num_wrong += 1

                if linkages_from_rd1 is False:
                    MarkerMAG_linkages_assessed_handle.write('%s\tWrong\n' % each_linkage.strip())
                else:
                    MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tS1\tWrong\n' % (id_16s, id_mag, link_num))

                if id_mag not in linkage_assessment_dict:
                    linkage_assessment_dict[id_mag] = ['Wrong']
                else:
                    linkage_assessment_dict[id_mag].append('Wrong')

                for each_16s_cluster in matched_cluster_16s:
                    current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                    for each_ref_1 in current_16s_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict.get(each_ref_1, 'NA')))

                for each_mag_cluster in matched_cluster_mag:
                    current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                    for each_ref_2 in current_mag_cluster_ref:
                        wrong_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_wrong, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict.get(each_ref_2, 'NA')))
                wrong_linkages_txt_handle.write('\n')
        else:
            linkage_num_unknown += 1

            if linkages_from_rd1 is False:
                MarkerMAG_linkages_assessed_handle.write('%s\tUnknown\n' % each_linkage.strip())
            else:
                MarkerMAG_linkages_assessed_handle.write('%s\t%s\t%s\tS1\tUnknown\n' % (id_16s, id_mag, link_num))

            if id_mag not in linkage_assessment_dict:
                linkage_assessment_dict[id_mag] = ['Unknown']
            else:
                linkage_assessment_dict[id_mag].append('Unknown')

            for each_16s_cluster in matched_cluster_16s:
                current_16s_cluster_ref = cluster_to_ref_dict[each_16s_cluster]
                for each_ref_1 in current_16s_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, '16S', each_16s_cluster, each_ref_1, ref_to_strain_dict.get(each_ref_1, 'NA')))
            for each_mag_cluster in matched_cluster_mag:
                current_mag_cluster_ref = cluster_to_ref_dict[each_mag_cluster]
                for each_ref_2 in current_mag_cluster_ref:
                    unknown_linkages_txt_handle.write('%s\t%s___%s\t%s\t%s\t%s\t%s\n' % (linkage_num_unknown, id_16s, id_mag, 'MAG', each_mag_cluster, each_ref_2, ref_to_strain_dict.get(each_ref_2, 'NA')))
            unknown_linkages_txt_handle.write('\n')
wrong_linkages_txt_handle.close()
unknown_linkages_txt_handle.close()
MarkerMAG_linkages_assessed_handle.close()


# get assess stats at mag level
linked_mag_dict_rd1 = {}
linked_mag_dict_rd2 = {}
total_linkage_num_rd1 = 0
total_linkage_num_rd2 = 0
correct_link_rd1 = 0
correct_link_rd2 = 0
wrong_link_rd1 = 0
wrong_link_rd2 = 0
unknown_link_rd1 = 0
unknown_link_rd2 = 0
for each_link in open(MarkerMAG_linkages_assessed):
    if not each_link.startswith('MarkerGene	GenomicSeq	Linkage	Round	Assessment'):
        each_link_split = each_link.strip().split('\t')
        gnm_id = each_link_split[1]
        linked_rd = each_link_split[3]
        assessment = each_link_split[4]
        if linked_rd == 'S1':
            total_linkage_num_rd1 += 1

            if assessment == 'Correct':
                correct_link_rd1 += 1
            if assessment == 'Wrong':
                wrong_link_rd1 += 1
            if assessment == 'Unknown':
                unknown_link_rd1 += 1

            if gnm_id not in linked_mag_dict_rd1:
                linked_mag_dict_rd1[gnm_id] = {assessment}
            else:
                linked_mag_dict_rd1[gnm_id].add(assessment)
        if linked_rd == 'S2':
            total_linkage_num_rd2 += 1

            if assessment == 'Correct':
                correct_link_rd2 += 1
            if assessment == 'Wrong':
                wrong_link_rd2 += 1
            if assessment == 'Unknown':
                unknown_link_rd2 += 1

            if gnm_id not in linked_mag_dict_rd1:
                linked_mag_dict_rd2[gnm_id] = {assessment}
            else:
                linked_mag_dict_rd2[gnm_id].add(assessment)


total_linkage_num_both = total_linkage_num_rd1 + total_linkage_num_rd2
total_linked_mag_num = len(linked_mag_dict_rd1) + len(linked_mag_dict_rd2)

rd1_correct_num, rd1_unknown_gnm, rd1_wrong_gnm, rd1_ambiguous_gnm = gnm_level_asessment(linked_mag_dict_rd1)
rd2_correct_num, rd2_unknown_gnm, rd2_wrong_gnm, rd2_ambiguous_gnm = gnm_level_asessment(linked_mag_dict_rd2)

correct_num_both        = rd1_correct_num + rd2_correct_num
unknown_gnm_both_num    = len(rd1_unknown_gnm) + len(rd2_unknown_gnm)
wrong_gnm_both_num      = len(rd1_wrong_gnm) + len(rd2_wrong_gnm)
ambiguous_gnm_both_num  = len(rd1_ambiguous_gnm) + len(rd2_ambiguous_gnm)

recovery_str_rd1   = '%s/%s(%s)' % (rd1_correct_num,  total_query_mag_num, float("{0:.2f}".format(rd1_correct_num*100/total_query_mag_num)))
recovery_str_rd2   = '%s/%s(%s)' % (rd2_correct_num,  total_query_mag_num, float("{0:.2f}".format(rd2_correct_num*100/total_query_mag_num)))
recovery_str_both  = '%s/%s(%s)' % (correct_num_both, total_query_mag_num, float("{0:.2f}".format(correct_num_both*100/total_query_mag_num)))

accuracy_str_rd1 = '0/0(0)'
if (len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)) > 0:
    accuracy_str_rd1 = '%s/%s(%s)' % (rd1_correct_num, (len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)), float("{0:.2f}".format(rd1_correct_num * 100 / (len(linked_mag_dict_rd1) - len(rd1_unknown_gnm)))))

accuracy_str_rd2 = '0/0(0)'
if (len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)) > 0:
    accuracy_str_rd2   = '%s/%s(%s)' % (rd2_correct_num,  (len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)), float("{0:.2f}".format(rd2_correct_num*100/(len(linked_mag_dict_rd2) - len(rd2_unknown_gnm)))))

accuracy_str_both = '0/0(0)'
if (total_linked_mag_num - unknown_gnm_both_num) > 0:
    accuracy_str_both  = '%s/%s(%s)' % (correct_num_both, (total_linked_mag_num - unknown_gnm_both_num), float("{0:.2f}".format(correct_num_both*100/(total_linked_mag_num - unknown_gnm_both_num))))

correct_link_both = correct_link_rd1 + correct_link_rd2
wrong_link_both   = wrong_link_rd1 + wrong_link_rd2
unknown_link_both = unknown_link_rd1 + unknown_link_rd2

accuracy_str_rd1_link_level = '0/0(0)'
if (total_linkage_num_rd1 - unknown_link_rd1) > 0:
    accuracy_str_rd1_link_level   = '%s/%s(%s)' % (correct_link_rd1,  (total_linkage_num_rd1 - unknown_link_rd1), float("{0:.2f}".format(correct_link_rd1*100/(total_linkage_num_rd1 - unknown_link_rd1))))


accuracy_str_rd2_link_level = '0/0(0)'
if (total_linkage_num_rd2 - unknown_link_rd2) > 0:
    accuracy_str_rd2_link_level   = '%s/%s(%s)' % (correct_link_rd2,  (total_linkage_num_rd2 - unknown_link_rd2), float("{0:.2f}".format(correct_link_rd2*100/(total_linkage_num_rd2 - unknown_link_rd2))))

accuracy_str_both_link_level = '0/0(0)'
if (total_linkage_num_both - unknown_link_both) > 0:
    accuracy_str_both_link_level  = '%s/%s(%s)' % (correct_link_both, (total_linkage_num_both - unknown_link_both), float("{0:.2f}".format(correct_link_both*100/(total_linkage_num_both - unknown_link_both))))

prefix = os.path.basename(MarkerMAG_linkages).split('_identified_linkages_genome_level')[0]
print('%s\tRound\t|\tLink\tYes\tNA\tNo\tAccuracy\t|\tMAG\tYes\tNA\tNo\tY/N\tRecovery\tAccuracy' % prefix)
print('%s\tRd_1\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_rd1,  correct_link_rd1,  unknown_link_rd1,  wrong_link_rd1,  accuracy_str_rd1_link_level,  len(linked_mag_dict_rd1), rd1_correct_num,  len(rd1_unknown_gnm), len(rd1_wrong_gnm), len(rd1_ambiguous_gnm), recovery_str_rd1, accuracy_str_rd1))
print('%s\tRd_2\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_rd2,  correct_link_rd2,  unknown_link_rd2,  wrong_link_rd2,  accuracy_str_rd2_link_level,  len(linked_mag_dict_rd2), rd2_correct_num,  len(rd2_unknown_gnm), len(rd2_wrong_gnm), len(rd2_ambiguous_gnm), recovery_str_rd2, accuracy_str_rd2))
print('%s\tBoth\t|\t%s\t%s\t%s\t%s\t%s\t|\t%s\t%s\t%s\t%s\t%s\t%s\t%s' % (prefix, total_linkage_num_both, correct_link_both, unknown_link_both, wrong_link_both, accuracy_str_both_link_level, total_linked_mag_num,      correct_num_both, unknown_gnm_both_num, wrong_gnm_both_num, ambiguous_gnm_both_num, recovery_str_both, accuracy_str_both))


'''
------------------------------------------------------------------------------------------------------------

# cami_hc_SILVA138_id99_50_subsample_100_3569
cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0512_tmp_MarkerMAG_wd_up/hc_0512_tmp_step_1_wd
grep -E '@|cami_hc_SILVA138_id99_50_subsample_100_3569' /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0511_very_sensitive_MarkerMAG_wd/hc_0511_very_sensitive_step_1_wd/hc_0511_very_sensitive_input_reads_to_16S_reformatted.sam > input_reads_to_16S_subset.sam

# cami_hc_14___NODE_13073_length_6967_cov_0.639487
cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0512_tmp_MarkerMAG_wd_up/hc_0512_tmp_step_1_wd
grep -E '@|cami_hc_14___C___NODE_13073_length_6967_cov_0.639487' rd1_extracted_to_gnm_reformatted.sam > rd1_extracted_to_gnm_subset.sam

------------------------------------------------------------------------------------------------------------


cami_hc_S1_24842618.2

cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0512_MarkerMAG_wd/hc_0512_step_1_wd
grep 'cami_hc_S1_24842618.' hc_0512_input_reads_to_16S_reformatted.sam > reads_to_16s_subset.sam
grep 'cami_hc_S1_24842618.' rd1_extracted_to_gnm_reformatted.sam > extracted_to_gnm_subset.sam

bowtie2-build --quiet --threads 12 -f cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_cbd.fa cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_cbd
bowtie2 -x cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_cbd -1 cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R1.fa -2 cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R2.fa -S test_p.sam -p 12 -f --local --all --no-unal

bowtie2 -x cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_cbd -U cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R1.fa,cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R2.fa -S test_up.sam -p 12 -f --local --all --no-unal

bowtie2 -x cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_cbd -U cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R1.fa,cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R2.fa -S test_up2.sam -p 12 -f --local --all --no-unal -N 1
bowtie2 -x cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_cbd -U cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R1.fa,cami_hc_SILVA138_id99_50_subsample_100_3770___cami_hc_14___NODE_13073_length_6967_cov_0.639487_R2.fa -S test_up_L30.sam -p 12 -f --local --all --no-unal -N 1 -L 30


------------------------------------------------------------------------------------------------------------

cami_hc_SILVA138_id99_50_subsample_100_2419	cami_hc_66	126	S1	Correct
cami_hc_SILVA138_id99_50_subsample_100_1159	cami_hc_66	69
cami_hc_SILVA138_id99_50_subsample_100_1198	cami_hc_66	69

cami_hc_SILVA138_id99_50_subsample_100_1159	cami_hc_66	41	S1	Wrong
cami_hc_SILVA138_id99_50_subsample_100_1198	cami_hc_66	41	S1	Wrong
cami_hc_SILVA138_id99_50_subsample_100_2419	cami_hc_66	29                  why much less this time?


why lost the following three linkages in the new run?
cami_hc_SILVA138_id99_50_subsample_100_2419___cami_hc_66___NODE_1112_length_54304_cov_1.219168
cami_hc_S2_9871673.1
cami_hc_S2_9871673.2

cami_hc_SILVA138_id99_50_subsample_100_2419___cami_hc_66___NODE_1393_length_45743_cov_1.217991
cami_hc_SILVA138_id99_50_subsample_100_2419___cami_hc_66___NODE_5764_length_14276_cov_1.210482

# check cami_hc_S2_9871673
cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0512_MarkerMAG_wd
grep 'cami_hc_S2_9871673.' ../hc_0511_very_sensitive_MarkerMAG_wd/hc_0511_very_sensitive_step_1_wd/hc_0511_very_sensitive_input_reads_to_16S_reformatted.sam > linked_to_16s.sam
grep 'cami_hc_S2_9871673.' hc_0512_step_1_wd/rd1_extracted_to_gnm_reformatted.sam > linked_to_gnm.sam

>cami_hc_S2_9871673.1
AATCAATAAAAACGTCAACGTTTTAGTTTTAAATCTTATCTGGCAGATGCCCGTCATCTGCAGATAAACAATTGAGCAAGTCAAACACTTTTTGGAGAGT
>cami_hc_S2_9871673.2
CTCACCCGTCCGCCGCTAACTTGAACGGAGCAAGCTCCGTCAAGTCCGCTCGACTTGCATGTATTAGGCACGCCGCCAGCGTTCGTCCTGAGCCAGGATC

cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0512_MarkerMAG_wd/hc_0512_step_1_wd/cami_hc_refined_bins_db

# got it
bowtie2-build --quiet --threads 12 -f cami_hc_66___NODE_1112_length_54304_cov_1.219168.fa cami_hc_66___NODE_1112_length_54304_cov_1.219168
bowtie2 -x cami_hc_66___NODE_1112_length_54304_cov_1.219168 -1 cami_hc_S2_9871673.1.fa -2 cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673.sam -p 12 -f --local --all --no-unal
# cami_hc_S2_9871673.1	89	cami_hc_66___C___NODE_1112_length_54304_cov_1.219168	1	255	50S50M	=	1	0	ACTCTCCAAAAAGTGTTTGACTTGCTCAATTGTTTATCTGCAGATGACGGGCATCTGCCAGATAAGATTTAAAACTAAAACGTTGACGTTTTTATTGATT	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII	AS:i:100	XN:i:0	XM:i:0	XO:i:0	XG:i:0	NM:i:0	MD:Z:50	YT:Z:UP

# not here !!!
bowtie2 -x cami_hc_refined_bins_combined -1 cami_hc_S2_9871673.1.fa -2 cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673_vs_all.sam -p 12 -f --local --all --no-unal

# not here !!!
bowtie2-build --quiet --threads 12 -f cami_hc_66.fasta cami_hc_66
bowtie2 -x cami_hc_66 -1 cami_hc_S2_9871673.1.fa -2 cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673_vs_hc_66_as_p.sam -p 12 -f --local --all --no-unal
bowtie2 -x cami_hc_66 -U cami_hc_S2_9871673.1.fa,cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673_vs_hc_66_as_up.sam -p 12 -f --local --all --no-unal
bowtie2 -x cami_hc_66 -1 cami_hc_S2_9871673.1.fa -2 cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673_vs_hc_66_as_p_all.sam -p 12 -f --local --all

bowtie2 -x cami_hc_refined_bins_renamed/cami_hc_66 -1 cami_hc_S2_9871673.1.fa -2 cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673_vs_hc_66_as_p_all.sam -p 12 -f --local --all


# got it
bowtie2-build --quiet --threads 12 -f Node_1112.fa Node_1112
bowtie2 -x Node_1112 -1 cami_hc_S2_9871673.1.fa -2 cami_hc_S2_9871673.2.fa -S cami_hc_S2_9871673_vs_1112.sam -p 12 -f --local --all --no-unal


# why not here
bowtie2 -x cami_hc_refined_bins_db/cami_hc_refined_bins_combined -1 rd1_extracted_R1.fasta -2 rd1_extracted_R2.fasta -S double_check.sam -p 12 -f --local --all --no-unal

# also not here
bowtie2 -x cami_hc_refined_bins_db/cami_hc_refined_bins_combined -1 rd1_extracted_R1.fasta -2 rd1_extracted_R2.fasta -S double_check_very_sensitive.sam -p 12 -f --local --all --no-unal --very-sensitive-local

bowtie2 -x cami_hc_refined_bins_db/cami_hc_refined_bins_combined -1 rd1_extracted_R1.fasta -2 rd1_extracted_R2.fasta -S double_check_super_sensitive.sam -p 12 -f --local --all --no-unal --very-sensitive-local



------------------------------------------------------------------------------------------------------------

cami_hc_S4_52228456.2       middle clip on      cami_hc_SILVA138_id99_75_subsample_75_3121      why?  A: not mapped, need to try very sensitive parameters



cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_05010_MarkerMAG_wd/hc_05010_step_1_wd/cami_hc_SILVA138_uclust_0.999_polished_index




cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0511_super_sensitive_MarkerMAG_wd/hc_0511_super_sensitive_step_1_wd
grep cami_hc_S4_52228456.2 hc_0511_super_sensitive_input_reads_to_16S_best_match.sam > super_sensitive_subset.sam

cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0511_very_sensitive_MarkerMAG_wd/hc_0511_very_sensitive_step_1_wd
grep cami_hc_S4_52228456.2 hc_0511_very_sensitive_input_reads_to_16S_best_match.sam > very_sensitive_subset.sam

grep cami_hc_S4_52228456.2 ../hc_05010_input_reads_to_16S.sam > vs_all_vs_all.sam


module load bowtie
cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_05010_MarkerMAG_wd/hc_05010_step_1_wd/cami_hc_SILVA138_uclust_0.999_polished_index
bowtie2-build --quiet --threads 12 -f cami_hc_SILVA138_id99_75_subsample_75_3121.fa cami_hc_SILVA138_id99_75_subsample_75_3121
bowtie2-build --quiet --threads 12 -f cami_hc_SILVA138_uclust_0.999_polished.fa cami_hc_SILVA138_uclust_0.999_polished
bowtie2-build --quiet --threads 12 -f Five_16s.fa Five_16s

BioSAK select_seq -seq cami_hc_SILVA138_uclust_0.999_polished.fa -id Ten_16s.txt -out Ten_16s.fa -option 1 -oneline
BioSAK select_seq -seq cami_hc_SILVA138_uclust_0.999_polished.fa -id Twenty_16s.txt -out Twenty_16s.fa -option 1 -oneline
bowtie2-build --quiet --threads 12 -f Ten_16s.fa Ten_16s
bowtie2-build --quiet --threads 12 -f Twenty_16s.fa Twenty_16s

bowtie2 -x cami_hc_SILVA138_uclust_0.999_polished -1 cami_hc_S4_52228456.1.fas -2 cami_hc_S4_52228456.2.fas -S vs_all_default.sam -p 12 -f --local --all --no-unal
bowtie2 -x cami_hc_SILVA138_id99_75_subsample_75_3121 -1 cami_hc_S4_52228456.1.fas -2 cami_hc_S4_52228456.2.fas -S vs_single_default.sam -p 12 -f --local --all --no-unal
bowtie2 -x Five_16s -1 cami_hc_S4_52228456.1.fas -2 cami_hc_S4_52228456.2.fas -S vs_Five_16s_default.sam -p 12 -f --local --all --no-unal

bowtie2 -x Ten_16s -1 cami_hc_S4_52228456.1.fas -2 cami_hc_S4_52228456.2.fas -S vs_Ten_16s_default.sam -p 12 -f --local --all --no-unal
bowtie2 -x Twenty_16s -1 cami_hc_S4_52228456.1.fas -2 cami_hc_S4_52228456.2.fas -S vs_Twenty_16s_default.sam -p 12 -f --local --all --no-unal


bowtie2 -x cami_hc_SILVA138_id99_75_subsample_75_3121 -1 cami_hc_S4_52228456.1.fas -2 cami_hc_S4_52228456.2.fas -S cami_hc_S4_52228456.sam -p 12 -f --local --all --no-unal



cami_hc_SILVA138_id99_50_subsample_100_2419	cami_hc_66	126	S1	Correct
cami_hc_SILVA138_id99_50_subsample_100_1159	cami_hc_66	69
cami_hc_SILVA138_id99_50_subsample_100_1198	cami_hc_66	69

cami_hc_SILVA138_id99_50_subsample_100_1159	cami_hc_66	41	S1	Wrong
cami_hc_SILVA138_id99_50_subsample_100_1198	cami_hc_66	41	S1	Wrong
cami_hc_SILVA138_id99_50_subsample_100_2419	cami_hc_66	29                  why much less this time?



hc_0514_stats_combined_filtered.txt	Round	|	Link	Yes	NA	No	Accuracy	|	MAG	Yes	NA	No	Y/N	Recovery	Accuracy
hc_0514_stats_combined_filtered.txt	Rd_1	|	93	87	0	6	87/93(93.55)	|	42	38	0	4	0	38/97(39.18)	38/42(90.48)
hc_0514_stats_combined_filtered.txt	Rd_2	|	0	0	0	0	0/0(0)	|	0	0	0	0	0	0/97(0.0)	0/0(0)
hc_0514_stats_combined_filtered.txt	Both	|	93	87	0	6	87/93(93.55)	|	42	38	0	4	0	38/97(39.18)	38/42(90.48)


hc_05010_stats_combined_filtered.txt	Rd_1	|	84	80	0	4	80/84(95.24)	|	40	36	0	4	0	36/97(37.11)	36/40(90.0)
hc_05010_stats_combined_filtered.txt	Rd_2	|	0	0	0	0	0/0(0)	        |	0	0	0	0	0	0/97(0.0)	    0/0(0)
hc_05010_stats_combined_filtered.txt	Both	|	84	80	0	4	80/84(95.24)	|	40	36	0	4	0	36/97(37.11)	36/40(90.0)

hc_0507_mis2_75_45_stats_combined_filtered.txt	Rd_1	|	39	33	1	5	33/38(86.84)	|	24	22	1	1	0	22/97(22.68)	22/23(95.65)
hc_0507_mis2_80_45_stats_combined_filtered.txt	Rd_1	|	45	36	1	8	36/44(81.82)	|	28	24	1	3	0	24/97(24.74)	24/27(88.89)
hc_0507_mis2_85_45_stats_combined_filtered.txt	Rd_1	|	45	36	1	8	36/44(81.82)	|	28	24	1	3	0	24/97(24.74)	24/27(88.89)

'''

'''
# cami_hc_S3_49165521.1

cami_hc_S5_42397362.1	{'cami_hc_SILVA138_id99_50_subsample_50_1792'}
cami_hc_S5_42397362.1	345	cami_hc_SILVA138_id99_75_subsample_75_2741	1	11	24S13=1X62=	=	1	0	CAGTTTTTCAACAGATTTTGTTGGAGAGTTTGATCCTCGCTCAGGACGAACGCTGGCGGCGTGCTTAACACATGCAAGTCGAGCGGAAAGGCCCTTCGGG	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII	AS:i:144	XN:i:0	XM:i:1	XO:i:0	XG:i:0	NM:i:1	MD:Z:13G62	YT:Z:UP
cami_hc_S5_42397362.1	345	cami_hc_SILVA138_id99_50_subsample_10_876	1	11	24S13=1X62=	=	1	0	CAGTTTTTCAACAGATTTTGTTGGAGAGTTTGATCCTCGCTCAGGACGAACGCTGGCGGCGTGCTTAACACATGCAAGTCGAGCGGAAAGGCCCTTCGGG	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII	AS:i:144	XN:i:0	XM:i:1	XO:i:0	XG:i:0	NM:i:1	MD:Z:13G62	YT:Z:UP
cami_hc_S5_42397362.1	89	cami_hc_SILVA138_id99_50_subsample_50_1792	1	11	16S21=1X62=	=	1	0	CAGTTTTTCAACAGATTTTGTTGGAGAGTTTGATCCTCGCTCAGGACGAACGCTGGCGGCGTGCTTAACACATGCAAGTCGAGCGGAAAGGCCCTTCGGG	IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII	AS:i:160	XN:i:0	XM:i:1	XO:i:0	XG:i:0	NM:i:1	MD:Z:21G62	YT:Z:UP

correct
cami_hc_SILVA138_id99_75_subsample_75_2741	cami_hc_3
cami_hc_SILVA138_id99_50_subsample_10_876	cami_hc_3
cami_hc_SILVA138_id99_50_subsample_50_1792  cami_hc_13

wrong
MarkerGene__cami_hc_SILVA138_id99_50_subsample_50_1792,GenomicSeq__cami_hc_3,534
MarkerGene__cami_hc_SILVA138_id99_75_subsample_75_2741,GenomicSeq__cami_hc_3,294
MarkerGene__cami_hc_SILVA138_id99_50_subsample_10_876,GenomicSeq__cami_hc_3,294
MarkerGene__cami_hc_SILVA138_id99_50_subsample_50_1792,GenomicSeq__cami_hc_13,72
MarkerGene__cami_hc_SILVA138_id99_50_subsample_10_876,GenomicSeq__cami_hc_13,43
MarkerGene__cami_hc_SILVA138_id99_75_subsample_75_2741,GenomicSeq__cami_hc_13,38

cd /srv/scratch/z5039045/MarkerMAG_wd/CAMI1/3_High_Complexity/hc_0505_mis2_75_45_keep_short_M_MarkerMAG_wd/hc_0505_mis2_75_45_keep_short_M_step_1_wd
grep cami_hc_SILVA138_id99_75_subsample_75_2741 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_1.sam
grep cami_hc_SILVA138_id99_50_subsample_10_876 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_2.sam
grep cami_hc_SILVA138_id99_50_subsample_50_1792 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_3.sam
cat sub_1.sam sub_2.sam sub_3.sam > sub.sam

grep cami_hc_SILVA138_id99_75_subsample_75_3138 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_4.sam
cat sub_1.sam sub_2.sam sub_3.sam sub_4.sam > sub.sam

grep cami_hc_SILVA138_id99_75_subsample_75_2741 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_1.sam
grep cami_hc_SILVA138_id99_50_subsample_10_876 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_2.sam
grep cami_hc_SILVA138_id99_50_subsample_50_1792 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S_best_match.sam > sub_3.sam
cat sub_1.sam sub_2.sam sub_3.sam > sub.sam


grep cami_hc_SILVA138_id99_75_subsample_75_2741 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S.sam > sub_1all.sam
grep cami_hc_SILVA138_id99_50_subsample_10_876 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S.sam > sub_2all.sam
grep cami_hc_SILVA138_id99_50_subsample_50_1792 hc_0505_mis2_75_45_keep_short_M_input_reads_to_16S.sam > sub_3all.sam
cat sub_1all.sam sub_2all.sam sub_3all.sam > sub_all.sam


wrong
cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3(534)	NODE_780_length_68394_cov_3.028655	115	0	0	S1
cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3(534)	NODE_208_length_155603_cov_3.027781	136	0	0	S1
cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3(534)	NODE_701_length_72282_cov_3.046022	185	0	0	S1
cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3(534)	NODE_1353_length_46895_cov_3.028037	98	0	0	S1
MarkerGene__cami_hc_SILVA138_id99_50_subsample_50_1792,GenomicSeq__cami_hc_3,534
MarkerGene__cami_hc_SILVA138_id99_75_subsample_75_2741,GenomicSeq__cami_hc_3,294
MarkerGene__cami_hc_SILVA138_id99_50_subsample_10_876,GenomicSeq__cami_hc_3,294


hc_0505_mis2_75_45_drop_short_M	Rd_1	|	56	49	2	5	49/54(90.74)	|	40	35	2	3	0	35/97(36.08)	35/38(92.11)
hc_0505_mis2_75_45_drop_short_M	Rd_2	|	5	4	0	1	4/5(80.0)	    |	5	4	0	1	0	4/97(4.12)	    4/5(80.0)
hc_0505_mis2_75_45_drop_short_M	Both	|	61	53	2	6	53/59(89.83)	|	45	39	2	4	0	39/97(40.21)	39/43(90.7)

hc_0505_mis2_75_45_keep_short_M	Rd_1	|	53	46	2	5	46/51(90.2)	    |	40	35	2	3	0	35/97(36.08)	35/38(92.11)
hc_0505_mis2_75_45_keep_short_M	Rd_2	|	8	4	2	2	4/6(66.67)	    |	6	3	1	2	0	3/97(3.09)	    3/5(60.0)
hc_0505_mis2_75_45_keep_short_M	Both	|	61	50	4	7	50/57(87.72)	|	46	38	3	5	0	38/97(39.18)	38/43(88.37)

hc_0505_mis2_75_45	    Rd_1	|	60	41	2	17	41/58(70.69)	|	37	30	2	5	0	30/97(30.93)	30/35(85.71)
hc_0505_mis2_75_45	    Rd_2	|	12	9	2	1	9/10(90.0)	    |	8	6	1	1	0	6/97(6.19)	    6/7(85.71)
hc_0505_mis2_75_45	    Both	|	72	50	4	18	50/68(73.53)	|	45	36	3	6	0	36/97(37.11)	36/42(85.71)

hc_0505_mis1.9_75_45	Rd_1	|	72	42	2	28	42/70(60.0)	    |	37	27	2	8	0	27/97(27.84)	27/35(77.14)
hc_0505_mis1.9_75_45	Rd_2	|	13	7	2	4	7/11(63.64)	    |	9	6	1	2	0	6/97(6.19)	    6/8(75.0)
hc_0505_mis1.9_75_45	Both	|	85	49	4	32	49/81(60.49)	|	46	33	3	10	0	33/97(34.02)	33/43(76.74)

hc_0505_mis1.8_75_45	Rd_1	|	73	42	3	28	42/70(60.0)	    |	37	27	2	8	0	27/97(27.84)	27/35(77.14)
hc_0505_mis1.8_75_45	Rd_2	|	15	7	2	6	7/13(53.85)	    |	10	6	1	3	0	6/97(6.19)	    6/9(66.67)
hc_0505_mis1.8_75_45	Both	|	88	49	5	34	49/83(59.04)	|	47	33	3	11	0	33/97(34.02)	33/44(75.0)


stats_combined_filtered.txt	Rd_1	|	57	55	2	0	55/55(100.0)	|	35	33	2	0	0	33/97(34.02)	33/33(100.0)
stats_combined_filtered.txt	Rd_1	|	55	53	2	0	53/53(100.0)	|	35	33	2	0	0	33/97(34.02)	33/33(100.0)


	                                                    Round	|	Link	Yes	NA	No	Accuracy	|	MAG	Yes	NA	No	Y/N	Recovery	    Accuracy
CAMI_hc_0428_stats_combined_filtered.txt	            Rd_1	|	49	41	4	4	41/45(91.11)	|	41	33	4	4	0	33/97(34.02)	33/37(89.19)
CAMI_hc_0428_specific_stats_combined_filtered.txt	    Rd_1	|	48	41	3	4	41/45(91.11)	|	40	33	3	4	0	33/97(34.02)	33/37(89.19)
CAMI_hc_0428_very_specific_stats_combined_filtered.txt	Rd_1	|	42	34	3	5	34/39(87.18)	|	38	30	3	5	0	30/97(30.93)	30/35(85.71)


1	cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3	16S	C246_1	tax_1137264	NA
1	cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3	16S	C246_1	tax_1169165	NA
1	cami_hc_SILVA138_id99_50_subsample_50_1792___cami_hc_3	MAG	C248_0	tax_134676	NA

2	cami_hc_SILVA138_id99_75_subsample_75_1629___cami_hc_13	16S	C229_0	tax_1051629	NA
2	cami_hc_SILVA138_id99_75_subsample_75_1629___cami_hc_13	MAG	C246_1	tax_1137264	NA
2	cami_hc_SILVA138_id99_75_subsample_75_1629___cami_hc_13	MAG	C246_1	tax_1169165	NA

3	cami_hc_SILVA138_id99_75_subsample_75_5112___cami_hc_18	16S	C103_1	tax_1124469	NA
3	cami_hc_SILVA138_id99_75_subsample_75_5112___cami_hc_18	16S	C103_1	tax_991945	NA
3	cami_hc_SILVA138_id99_75_subsample_75_5112___cami_hc_18	16S	C103_1	tax_991968	NA
3	cami_hc_SILVA138_id99_75_subsample_75_5112___cami_hc_18	MAG	C100_0	tax_349966	NA

4	cami_hc_SILVA138_id99_50_subsample_50_2324___cami_hc_11	16S	C259_1	tax_1160164	NA
4	cami_hc_SILVA138_id99_50_subsample_50_2324___cami_hc_11	16S	C259_1	tax_1328339	NA
4	cami_hc_SILVA138_id99_50_subsample_50_2324___cami_hc_11	16S	C259_1	tax_520453	NA
4	cami_hc_SILVA138_id99_50_subsample_50_2324___cami_hc_11	MAG	C277_0	tax_1287182	NA

'''



from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord


def get_max_clp_and_index(r1_cigar_list, r2_cigar_list):

    r1_cigar_list_split = [cigar_splitter(i) for i in r1_cigar_list]
    r2_cigar_list_split = [cigar_splitter(i) for i in r2_cigar_list]

    r1_cigar_list_split_only_clp = []
    for each_r1_cigar_split in r1_cigar_list_split:
        clp_len_l = 0
        if each_r1_cigar_split[0][-1] in ['S', 's']:
            clp_len_l = int(each_r1_cigar_split[0][:-1])
        clp_len_r = 0
        if each_r1_cigar_split[-1][-1] in ['S', 's']:
            clp_len_r = int(each_r1_cigar_split[-1][:-1])
        r1_cigar_list_split_only_clp.append([clp_len_l, clp_len_r])

    r2_cigar_list_split_only_clp = []
    for each_r2_cigar_split in r2_cigar_list_split:
        clp_len_l = 0
        if each_r2_cigar_split[0][-1] in ['S', 's']:
            clp_len_l = int(each_r2_cigar_split[0][:-1])
        clp_len_r = 0
        if each_r2_cigar_split[-1][-1] in ['S', 's']:
            clp_len_r = int(each_r2_cigar_split[-1][:-1])
        r2_cigar_list_split_only_clp.append([clp_len_l, clp_len_r])

    cigar_list_split_only_clp_r1_r2 = [r1_cigar_list_split_only_clp, r2_cigar_list_split_only_clp]

    max_value = 0
    max_value_index = ''
    for num_list_1 in cigar_list_split_only_clp_r1_r2[0]:
        if num_list_1[0] > max_value:
            max_value = num_list_1[0]
            max_value_index = 'r1_l'
        if num_list_1[1] > max_value:
            max_value = num_list_1[1]
            max_value_index = 'r1_r'
    for num_list_2 in cigar_list_split_only_clp_r1_r2[1]:
        if num_list_2[0] > max_value:
            max_value = num_list_2[0]
            max_value_index = 'r2_l'
        if num_list_2[1] > max_value:
            max_value = num_list_2[1]
            max_value_index = 'r2_r'

    # get the best cigar
    best_cigar = ''
    if max_value_index == 'r1_l':
        if best_cigar == '':
            for each_cigar in r1_cigar_list:
                if (each_cigar.startswith('%sS' % max_value)) or (each_cigar.startswith('%ss' % max_value)):
                    best_cigar = each_cigar
    elif max_value_index == 'r1_r':
        if best_cigar == '':
            for each_cigar in r1_cigar_list:
                if (each_cigar.endswith('%sS' % max_value)) or (each_cigar.endswith('%ss' % max_value)):
                    best_cigar = each_cigar
    elif max_value_index == 'r2_l':
        if best_cigar == '':
            for each_cigar in r2_cigar_list:
                if (each_cigar.startswith('%sS' % max_value)) or (each_cigar.startswith('%ss' % max_value)):
                    best_cigar = each_cigar
    elif max_value_index == 'r2_r':
        if best_cigar == '':
            for each_cigar in r2_cigar_list:
                if (each_cigar.endswith('%sS' % max_value)) or (each_cigar.endswith('%ss' % max_value)):
                    best_cigar = each_cigar

    return best_cigar, max_value, max_value_index


def get_rc(seq_in):
    seq_in_rc = str(SeqRecord(Seq(seq_in)).reverse_complement().seq)
    return seq_in_rc


def sam_flag_to_rc(flag_value):

    read_rced = 'na'
    if flag_value != '':
        binary_flag = "{0:b}".format(int(flag_value))
        binary_flag_len = len(str(binary_flag))
        binary_flag_polished = '0' * (12 - binary_flag_len) + str(binary_flag)

        if binary_flag_polished[7] == '0':
            read_rced = False
        if binary_flag_polished[7] == '1':
            read_rced = True

    return read_rced


class MappingRecord:

    #  sequences store in r1_seq and r2_seq should NOT been reverse complemented

    def __init__(self):

        self.r1_seq = ''
        self.r2_seq = ''
        self.r1_seq_qual = '*'
        self.r2_seq_qual = '*'

        self.r1_refs = dict()
        self.r2_refs = dict()

        self.r1_cigar_to_flag = dict()
        self.r2_cigar_to_flag = dict()

        self.r1_longest_clp_cigar = ''
        self.r1_longest_clp_falg  = ''
        self.r2_longest_clp_cigar = ''
        self.r2_longest_clp_falg  = ''

        self.qualified_reads           = False
        self.consider_round_2          = False
        self.consider_r1_unmapped_mate = False
        self.consider_r1_clipping_part = False
        self.consider_r2_unmapped_mate = False
        self.consider_r2_clipping_part = False

        self.r1_filtered_refs = set()
        self.r2_filtered_refs = set()

        self.r1_clipping_seq = ''
        self.r2_clipping_seq = ''

        self.r1_clipping_seq_qual = '*'
        self.r2_clipping_seq_qual = '*'

        self.unmapped_r1_refs = set()
        self.unmapped_r2_refs = set()

        self.clipping_r1_refs = set()
        self.clipping_r2_refs = set()


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if (each_element.isalpha() is True) or (each_element == '='):
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def get_cigar_stats(cigar_splitted):

    # aligned_len: M I X =
    # clipping_len: S
    # mismatch_len: X I D
    # mismatch_pct = mismatch_len / aligned_len
    # aligned_pct  = aligned_len  / (aligned_len + clipping_len)
    # clipping_pct = clipping_len / (aligned_len + clipping_len)

    aligned_len = 0
    clipping_len = 0
    mismatch_len = 0
    for each_part in cigar_splitted:
        each_part_len = int(each_part[:-1])
        each_part_cate = each_part[-1]

        # get aligned_len
        if each_part_cate in {'M', 'm', 'I', 'i', 'X', 'x', '='}:
            aligned_len += each_part_len

        # get clipping_len
        if each_part_cate in ['S', 's']:
            clipping_len += each_part_len

        # get mismatch_len
        if each_part_cate in {'I', 'i', 'X', 'x', 'D', 'd'}:
            mismatch_len += each_part_len

    aligned_pct  = float("{0:.2f}".format(aligned_len * 100 / (aligned_len + clipping_len)))
    clipping_pct = float("{0:.2f}".format(clipping_len * 100 / (aligned_len + clipping_len)))
    mismatch_pct = float("{0:.2f}".format(mismatch_len * 100 / (aligned_len)))

    return aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct


wd          = '/Users/songweizhi/Desktop/round2'
sam_file    = '%s/step_1_unlinked_combined_gnms_ends_1000bp.sam'    % wd
sam_file    = '%s/subset.sam'                                       % wd
round2_fq   = False


free_living_ext = 'fa'
if round2_fq is True:
    free_living_ext = 'fq'

free_living_ctg_R1 = '/Users/songweizhi/Desktop/new_algorithm_Kelp/round2_free_living_ctg_R1.%s' % free_living_ext
free_living_ctg_R2 = '/Users/songweizhi/Desktop/new_algorithm_Kelp/round2_free_living_ctg_R2.%s' % free_living_ext
free_living_ctg_UP = '/Users/songweizhi/Desktop/new_algorithm_Kelp/round2_free_living_ctg_UP.%s' % free_living_ext
min_M_len                               = 30
min_clp_len_round2                      = 30
max_mis_pct                             = 3
end_seq_len                             = 1000  # bp


round_2_MappingRecord_dict = {}
for each_line in open(sam_file):
    each_line_split = each_line.strip().split('\t')
    if not each_line.startswith('@'):
        store_read_seq = False
        read_id = each_line_split[0]
        read_id_base = '.'.join(read_id.split('.')[:-1])
        read_strand = read_id.split('.')[-1]
        read_flag = int(each_line_split[1])
        cigar = each_line_split[5]
        read_seq = each_line_split[9]
        read_seq_qual = each_line_split[10]
        if cigar != '*':
            ref_id = each_line_split[2]
            ref_pos = each_line_split[3]
            ref_id_with_pos = '%s_pos_%s' % (ref_id, ref_pos)
            cigar_splitted = cigar_splitter(cigar)
            aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct = get_cigar_stats(cigar_splitted)
            if mismatch_pct <= max_mis_pct:

                if read_id_base not in round_2_MappingRecord_dict:
                    round_2_MappingRecord_dict[read_id_base] = MappingRecord()

                if read_strand == '1':
                    round_2_MappingRecord_dict[read_id_base].r1_refs[ref_id_with_pos] = cigar
                    round_2_MappingRecord_dict[read_id_base].r1_cigar_to_flag[cigar] = read_flag

                if read_strand == '2':
                    round_2_MappingRecord_dict[read_id_base].r2_refs[ref_id_with_pos] = cigar
                    round_2_MappingRecord_dict[read_id_base].r2_cigar_to_flag[cigar] = read_flag

                if clipping_len >= min_clp_len_round2:
                    store_read_seq = True
            else:
                store_read_seq = True
        else:
            store_read_seq = True

        # store_read_seq into dict
        if store_read_seq is True:

            # turn back if read reverse complemented
            read_rc = sam_flag_to_rc(read_flag)
            read_seq_to_store = read_seq
            read_seq_qual_to_store = read_seq_qual
            if read_rc is True:
                read_seq_to_store = get_rc(read_seq)
                read_seq_qual_to_store = read_seq_qual[::-1]

            if read_id_base not in round_2_MappingRecord_dict:
                round_2_MappingRecord_dict[read_id_base] = MappingRecord()
            if read_strand == '1':
                if round_2_MappingRecord_dict[read_id_base].r1_seq == '':
                    round_2_MappingRecord_dict[read_id_base].r1_seq = read_seq_to_store
                    round_2_MappingRecord_dict[read_id_base].r1_seq_qual = read_seq_qual_to_store
            if read_strand == '2':
                if round_2_MappingRecord_dict[read_id_base].r2_seq == '':
                    round_2_MappingRecord_dict[read_id_base].r2_seq = read_seq_to_store
                    round_2_MappingRecord_dict[read_id_base].r2_seq_qual = read_seq_qual_to_store


# parse round_2_MappingRecord_dict
free_living_ctg_R1_handle = open(free_living_ctg_R1, 'w')
free_living_ctg_R2_handle = open(free_living_ctg_R2, 'w')
free_living_ctg_UP_handle = open(free_living_ctg_UP, 'w')
for read_basename in round_2_MappingRecord_dict.copy():
    read_mr = round_2_MappingRecord_dict[read_basename]
    r1_ref_cigar_list = list(read_mr.r1_refs.values())
    r2_ref_cigar_list = list(read_mr.r2_refs.values())
    best_cigar, max_clp, max_clp_location = get_max_clp_and_index(r1_ref_cigar_list, r2_ref_cigar_list)

    if (read_mr.r1_refs == {}) and (read_mr.r2_refs != {}):

        if len(read_mr.r2_refs) == 1:

            r2_ref_cigar    = r2_ref_cigar_list[0]
            r2_ref_flag     = read_mr.r2_cigar_to_flag[r2_ref_cigar]
            r2_ref_cigar_rc = sam_flag_to_rc(r2_ref_flag)

            # consider the unmapped mate only
            if max_clp < min_clp_len_round2:
                read_mr.consider_round_2 = True
                read_mr.consider_r2_unmapped_mate = True

                # write out sequence
                if round2_fq is False:
                    free_living_ctg_UP_handle.write('>%s.1\n' % read_basename)
                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_seq)
                else:
                    free_living_ctg_UP_handle.write('@%s.1\n' % read_basename)
                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_seq)
                    free_living_ctg_UP_handle.write('+\n')
                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_seq_qual)

            # consider both of unmapped mate and clipping part
            else:
                r2_ref_no_pos = list(read_mr.r2_refs.keys())[0].split('_pos_')[0]
                r2_ref_pos    = int(list(read_mr.r2_refs.keys())[0].split('_pos_')[1])

                if (r2_ref_no_pos[-1]) == (max_clp_location[-1]) == 'l':
                    read_mr.consider_round_2            = True
                    read_mr.consider_r2_unmapped_mate   = True
                    read_mr.consider_r2_clipping_part   = True

                    if r2_ref_cigar_rc is False:
                        read_mr.r2_clipping_seq          = read_mr.r2_seq[:max_clp]
                        if read_mr.r2_seq_qual != '*':
                            read_mr.r2_clipping_seq_qual = read_mr.r2_seq_qual[:max_clp]

                    if r2_ref_cigar_rc is True:
                        r2_seq_rc = get_rc(read_mr.r2_seq)
                        r2_clipping_seq_rc = r2_seq_rc[:max_clp]
                        read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)
                        if read_mr.r2_seq_qual != '*':
                            r2_seq_rc_qual = read_mr.r2_seq_qual[::-1]
                            r2_clipping_seq_rc_qual = r2_seq_rc_qual[:max_clp]
                            read_mr.r2_clipping_seq_qual = r2_clipping_seq_rc_qual[::-1]

                elif (r2_ref_no_pos[-1]) == (max_clp_location[-1]) == 'r':
                    read_mr.consider_round_2            = True
                    read_mr.consider_r2_unmapped_mate   = True
                    read_mr.consider_r2_clipping_part   = True

                    if r2_ref_cigar_rc is False:
                        read_mr.r2_clipping_seq          = read_mr.r2_seq[-max_clp:]
                        if read_mr.r2_seq_qual != '*':
                            read_mr.r2_clipping_seq_qual = read_mr.r2_seq_qual[-max_clp:]

                    if r2_ref_cigar_rc is True:
                        r2_seq_rc = get_rc(read_mr.r2_seq)
                        r2_clipping_seq_rc = r2_seq_rc[-max_clp:]
                        read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)
                        if read_mr.r2_seq_qual != '*':
                            r2_seq_rc_qual = read_mr.r2_seq_qual[::-1]
                            r2_clipping_seq_rc_qual = r2_seq_rc_qual[-max_clp:]
                            read_mr.r2_clipping_seq_qual = r2_clipping_seq_rc_qual[::-1]

                else:  # mapped to unwanted end, ignore
                    round_2_MappingRecord_dict.pop(read_basename)

                # write out sequence
                if read_mr.consider_round_2 is True:

                    if round2_fq is False:
                        # write out R1 fa
                        free_living_ctg_R1_handle.write('>%s.1\n' % read_basename)
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_seq)
                        # write out R2 fa
                        free_living_ctg_R2_handle.write('>%s.2\n' % read_basename)
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_clipping_seq)
                    else:
                        # write out R1 fq
                        free_living_ctg_R1_handle.write('@%s.1\n' % read_basename)
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_seq)
                        free_living_ctg_R1_handle.write('+\n')
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_seq_qual)
                        # write out R2 fq
                        free_living_ctg_R2_handle.write('@%s.2\n' % read_basename)
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_clipping_seq)
                        free_living_ctg_R2_handle.write('+\n')
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_clipping_seq_qual)

                # print('r1_clipping_seq: %s' % read_mr.r1_clipping_seq)
                # print('r1_clipping_seq: %s' % read_mr.r1_clipping_seq_qual)
                # print('r2_clipping_seq: %s' % read_mr.r2_clipping_seq)
                # print('r2_clipping_seq: %s' % read_mr.r2_clipping_seq_qual)
                # print()

        else:  # r2 mapped to multiple refs, ignore
            round_2_MappingRecord_dict.pop(read_basename)

    elif (read_mr.r1_refs != {}) and (read_mr.r2_refs == {}):

        if len(read_mr.r1_refs) == 1:

            r1_ref_cigar    = r1_ref_cigar_list[0]
            r1_ref_flag     = read_mr.r1_cigar_to_flag[r1_ref_cigar]
            r1_ref_cigar_rc = sam_flag_to_rc(r1_ref_flag)

            # consider the unmapped mate only
            if max_clp < min_clp_len_round2:
                read_mr.consider_round_2 = True
                read_mr.consider_r1_unmapped_mate = True

                # write out sequence
                if round2_fq is False:
                    free_living_ctg_UP_handle.write('>%s.2\n' % read_basename)
                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_seq)
                else:
                    free_living_ctg_UP_handle.write('@%s.2\n' % read_basename)
                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_seq)
                    free_living_ctg_UP_handle.write('+\n')
                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_seq_qual)

            # consider both of unmapped mate and clipping part
            else:
                r1_ref_no_pos = list(read_mr.r1_refs.keys())[0].split('_pos_')[0]
                r1_ref_pos    = int(list(read_mr.r1_refs.keys())[0].split('_pos_')[1])

                if (r1_ref_no_pos[-1]) == (max_clp_location[-1]) == 'l':
                    read_mr.consider_round_2            = True
                    read_mr.consider_r1_unmapped_mate   = True
                    read_mr.consider_r1_clipping_part   = True

                    if r1_ref_cigar_rc is False:
                        read_mr.r1_clipping_seq          = read_mr.r1_seq[:max_clp]
                        if read_mr.r1_seq_qual != '*':
                            read_mr.r1_clipping_seq_qual = read_mr.r1_seq_qual[:max_clp]

                    if r1_ref_cigar_rc is True:
                        r1_seq_rc = get_rc(read_mr.r1_seq)
                        r1_clipping_seq_rc = r1_seq_rc[:max_clp]
                        read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)
                        if read_mr.r1_seq_qual != '*':
                            r1_seq_rc_qual = read_mr.r1_seq_qual[::-1]
                            r1_clipping_seq_rc_qual = r1_seq_rc_qual[:max_clp]
                            read_mr.r1_clipping_seq_qual = r1_clipping_seq_rc_qual[::-1]

                elif (r1_ref_no_pos[-1]) == (max_clp_location[-1]) == 'r':
                    read_mr.consider_round_2            = True
                    read_mr.consider_r1_unmapped_mate   = True
                    read_mr.consider_r1_clipping_part   = True

                    if r1_ref_cigar_rc is False:
                        read_mr.r1_clipping_seq          = read_mr.r1_seq[-max_clp:]
                        if read_mr.r1_seq_qual != '*':
                            read_mr.r1_clipping_seq_qual = read_mr.r1_seq_qual[-max_clp:]

                    if r1_ref_cigar_rc is True:
                        r1_seq_rc = get_rc(read_mr.r1_seq)
                        r1_clipping_seq_rc = r1_seq_rc[-max_clp:]
                        read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)
                        if read_mr.r1_seq_qual != '*':
                            r1_seq_rc_qual = read_mr.r1_seq_qual[::-1]
                            r1_clipping_seq_rc_qual = r1_seq_rc_qual[-max_clp:]
                            read_mr.r1_clipping_seq_qual = r1_clipping_seq_rc_qual[::-1]

                else:  # mapped to unwanted end, ignore
                    round_2_MappingRecord_dict.pop(read_basename)

                # write out sequence
                if read_mr.consider_round_2 is True:

                    if round2_fq is False:
                        # write out R1 fa
                        free_living_ctg_R1_handle.write('>%s.1\n' % read_basename)
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_clipping_seq)
                        # write out R2 fa
                        free_living_ctg_R2_handle.write('>%s.2\n' % read_basename)
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_seq)
                    else:
                        # write out R1 fq
                        free_living_ctg_R1_handle.write('@%s.1\n' % read_basename)
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_clipping_seq)
                        free_living_ctg_R1_handle.write('+\n')
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_clipping_seq_qual)
                        # write out R2 fq
                        free_living_ctg_R2_handle.write('@%s.2\n' % read_basename)
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_seq)
                        free_living_ctg_R2_handle.write('+\n')
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_seq_qual)

        else:  # r1 mapped to multiple refs, ignore
            round_2_MappingRecord_dict.pop(read_basename)

    elif (read_mr.r1_refs != {}) and (read_mr.r2_refs != {}):
        if max_clp >= min_M_len:
            if (len(read_mr.r1_refs) == 1) and (len(read_mr.r2_refs) == 1):
                r1_ref_no_pos = list(read_mr.r1_refs.keys())[0].split('_pos_')[0]
                r2_ref_no_pos = list(read_mr.r2_refs.keys())[0].split('_pos_')[0]
                r1_ref_pos    = int(list(read_mr.r1_refs.keys())[0].split('_pos_')[1])
                r2_ref_pos    = int(list(read_mr.r2_refs.keys())[0].split('_pos_')[1])
                r1_ref_cigar  = list(read_mr.r1_refs.values())[0]
                r2_ref_cigar  = list(read_mr.r2_refs.values())[0]

                if r1_ref_no_pos == r2_ref_no_pos:
                    if (r1_ref_no_pos[-1]) == (r2_ref_no_pos[-1]) == (max_clp_location[-1]):

                        # print('%s r1_seq: %s'  % (read_basename, read_mr.r1_seq))
                        # print('%s r2_seq: %s'  % (read_basename, read_mr.r2_seq))
                        # print('%s r1_refs: %s' % (read_basename, read_mr.r1_refs))
                        # print('%s r2_refs: %s' % (read_basename, read_mr.r2_refs))
                        # print('%s r1_cigar_list: %s' % (read_basename, r1_ref_cigar_list))
                        # print('%s r1_cigar_to_flag: %s' % (read_basename, read_mr.r1_cigar_to_flag))
                        # print('%s r2_cigar_list: %s' % (read_basename, r2_ref_cigar_list))
                        # print('%s r2_cigar_to_flag: %s' % (read_basename, read_mr.r2_cigar_to_flag))
                        # print()
                        #print(r2_ref_flag)
                        #print(r2_ref_cigar_rc)

                        if (max_clp_location == 'r1_l') and (r1_ref_pos <= 5):
                            read_mr.consider_round_2 = True
                            read_mr.consider_r1_clipping_part = True
                            best_cigar_flag = read_mr.r1_cigar_to_flag[best_cigar]
                            best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                            if best_cigar_rc is False:
                                read_mr.r1_clipping_seq = read_mr.r1_seq[:max_clp]
                                if read_mr.r1_seq_qual != '*':
                                    read_mr.r1_clipping_seq_qual = read_mr.r1_seq_qual[:max_clp]
                            if best_cigar_rc is True:
                                r1_seq_rc = get_rc(read_mr.r1_seq)
                                r1_clipping_seq_rc = r1_seq_rc[:max_clp]
                                read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)
                                if read_mr.r1_seq_qual != '*':
                                    r1_seq_rc_qual = read_mr.r1_seq_qual[::-1]
                                    r1_clipping_seq_rc_qual = r1_seq_rc_qual[:max_clp]
                                    read_mr.r1_clipping_seq_qual = r1_clipping_seq_rc_qual[::-1]

                        elif (max_clp_location == 'r2_l') and (r2_ref_pos <= 5):
                            read_mr.consider_round_2 = True
                            read_mr.consider_r2_clipping_part = True
                            best_cigar_flag = read_mr.r2_cigar_to_flag[best_cigar]
                            best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                            if best_cigar_rc is False:
                                read_mr.r2_clipping_seq = read_mr.r2_seq[:max_clp]
                                if read_mr.r2_seq_qual != '*':
                                    read_mr.r2_clipping_seq_qual = read_mr.r2_seq_qual[:max_clp]
                            if best_cigar_rc is True:
                                r2_seq_rc = get_rc(read_mr.r2_seq)
                                r2_clipping_seq_rc = r2_seq_rc[:max_clp]
                                read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)
                                if read_mr.r2_seq_qual != '*':
                                    r2_seq_rc_qual = read_mr.r2_seq_qual[::-1]
                                    r2_clipping_seq_rc_qual = r2_seq_rc_qual[:max_clp]
                                    read_mr.r2_clipping_seq_qual = r2_clipping_seq_rc_qual[::-1]

                        elif (max_clp_location == 'r1_r') and (r1_ref_pos >= (end_seq_len/2)):
                            read_mr.consider_round_2 = True
                            read_mr.consider_r1_clipping_part = True
                            best_cigar_flag = read_mr.r1_cigar_to_flag[best_cigar]
                            best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                            if best_cigar_rc is False:
                                read_mr.r1_clipping_seq = read_mr.r1_seq[-max_clp:]
                                if read_mr.r1_seq_qual != '*':
                                    read_mr.r1_clipping_seq_qual = read_mr.r1_seq_qual[-max_clp:]
                            if best_cigar_rc is True:
                                r1_seq_rc = get_rc(read_mr.r1_seq)
                                r1_clipping_seq_rc = r1_seq_rc[-max_clp:]
                                read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)
                                if read_mr.r1_seq_qual != '*':
                                    r1_seq_rc_qual = read_mr.r1_seq_qual[::-1]
                                    r1_clipping_seq_rc_qual = r1_seq_rc_qual[-max_clp:]
                                    read_mr.r1_clipping_seq_qual = r1_clipping_seq_rc_qual[::-1]

                        elif (max_clp_location == 'r2_r') and (r2_ref_pos >= (end_seq_len/2)):
                            read_mr.consider_round_2 = True
                            read_mr.consider_r2_clipping_part = True
                            best_cigar_flag = read_mr.r2_cigar_to_flag[best_cigar]
                            best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                            if best_cigar_rc is False:
                                read_mr.r2_clipping_seq = read_mr.r2_seq[-max_clp:]
                                if read_mr.r2_seq_qual != '*':
                                    read_mr.r2_clipping_seq_qual = read_mr.r2_seq_qual[-max_clp:]
                            if best_cigar_rc is True:
                                r2_seq_rc = get_rc(read_mr.r2_seq)
                                r2_clipping_seq_rc = r2_seq_rc[-max_clp:]
                                read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)
                                if read_mr.r2_seq_qual != '*':
                                    r2_seq_rc_qual = read_mr.r2_seq_qual[::-1]
                                    r2_clipping_seq_rc_qual = r2_seq_rc_qual[-max_clp:]
                                    read_mr.r2_clipping_seq_qual = r2_clipping_seq_rc_qual[::-1]

                        else:  # not too many of them, ignore now, maybe worth check later,
                            round_2_MappingRecord_dict.pop(read_basename)
                            # print('%s\tr1_refs:\t%s\t%s' % (read_basename, read_mr.r1_refs, read_mr.r1_seq))
                            # print('%s\tr2_refs:\t%s\t%s' % (read_basename, read_mr.r2_refs, read_mr.r2_seq))

                        # write out sequence
                        if read_mr.consider_round_2 is True:

                            if read_mr.consider_r1_clipping_part is True:
                                if round2_fq is False:
                                    free_living_ctg_UP_handle.write('>%s.1\n' % read_basename)
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_clipping_seq)
                                else:
                                    free_living_ctg_UP_handle.write('@%s.1\n' % read_basename)
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_clipping_seq)
                                    free_living_ctg_UP_handle.write('+\n')
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_clipping_seq_qual)

                            if read_mr.consider_r2_clipping_part is True:
                                if round2_fq is False:
                                    free_living_ctg_UP_handle.write('>%s.2\n' % read_basename)
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_clipping_seq)
                                else:
                                    free_living_ctg_UP_handle.write('@%s.2\n' % read_basename)
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_clipping_seq)
                                    free_living_ctg_UP_handle.write('+\n')
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_clipping_seq_qual)

                    else:  # mapped to unwanted end, ignore
                        round_2_MappingRecord_dict.pop(read_basename)

                else:  # r1 and r2 mapped to different refs
                    # not too many of them, ignore now
                    round_2_MappingRecord_dict.pop(read_basename)

            else:  # r1 or r2 mapped to multiple refs
                # not too many of them, ignore now
                round_2_MappingRecord_dict.pop(read_basename)

        else:  # ignore and remove element from dict
            round_2_MappingRecord_dict.pop(read_basename)

    else:  # ignore and remove element from dict
        round_2_MappingRecord_dict.pop(read_basename)

free_living_ctg_R1_handle.close()
free_living_ctg_R2_handle.close()
free_living_ctg_UP_handle.close()






def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if each_element.isalpha() is True:
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


sam_file_mini_assembly  = '/Users/songweizhi/Desktop/scaffolds.sam'

gap_seq_to_reads_dict = {}
for each_line in open(sam_file_mini_assembly):
    if not each_line.startswith('@'):
        each_line_split = each_line.strip().split('\t')
        read_id = each_line_split[0]
        ref_id = each_line_split[2]
        cigar = each_line_split[5]
        cigar_splitted = cigar_splitter(cigar)

        qualified_mapping = False
        if (len(cigar_splitted) == 1) and (cigar[-1] == 'M'):
            qualified_mapping = True
            print(each_line_split)
        # allow one mismatch
        elif (len(cigar_splitted) == 3) and (cigar_splitted[0][-1] == 'M') and (cigar_splitted[2][-1] == 'M'):
            left_m_len = int(cigar_splitted[0][:-1])
            mismatch_len = int(cigar_splitted[1][:-1])
            right_m_len = int(cigar_splitted[2][:-1])
            read_len = left_m_len + right_m_len + mismatch_len
            left_m_pct = left_m_len*100/read_len
            right_m_pct = right_m_len*100/read_len
            if (left_m_len >= 30) and (right_m_len >= 30) and (left_m_pct >= 25) and (right_m_pct >= 25) and (mismatch_len <= 1):
                qualified_mapping = True

            else:
                pass
                #print(each_line_split)

        # elif len(cigar_splitted) == 4:



        if qualified_mapping is True:
            if ref_id not in gap_seq_to_reads_dict:
                gap_seq_to_reads_dict[ref_id] = [read_id]
            else:
                gap_seq_to_reads_dict[ref_id].append(read_id)




# print(gap_seq_to_reads_dict)
# print(len(gap_seq_to_reads_dict))



from Bio import SeqIO


def get_regions_to_ignore_from_barrnap_output(combined_barrnap_gff, ctg_len_dict):

    ctg_ignore_region_dict = {}

    for each_line in open(combined_barrnap_gff):
        if (not each_line.startswith('#')) and ('16S_rRNA' in each_line):
            each_line_split = each_line.strip().split('\t')
            ctg_id = each_line_split[0]
            ctg_len = ctg_len_dict[ctg_id]
            start_pos = int(each_line_split[3])
            end_pos = int(each_line_split[4])
            len_16s = end_pos - start_pos + 1
            left_gap = start_pos - 1
            right_gap = ctg_len - end_pos - 1

            if left_gap <= 50:
                if ctg_id not in ctg_ignore_region_dict:
                    ctg_ignore_region_dict[ctg_id] = {'left_end'}
                else:
                    ctg_ignore_region_dict[ctg_id].add('left_end')

            if right_gap <= 50:
                if ctg_id not in ctg_ignore_region_dict:
                    ctg_ignore_region_dict[ctg_id] = {'right_end'}
                else:
                    ctg_ignore_region_dict[ctg_id].add('right_end')

    return ctg_ignore_region_dict


def get_unlinked_mag_end_seq(ref_in, ref_in_end_seq, end_seq_len, ctg_ignore_region_dict_rd1):

    ctg_ignore_region_dict_rd2 = dict()

    # get ref seqs subset
    ref_subset_handle = open(ref_in_end_seq, 'w')
    for ref_seq in SeqIO.parse(ref_in, 'fasta'):

        ref_seq_id = ref_seq.id
        ref_seq_len = len(ref_seq.seq)

        if ref_seq_len < end_seq_len * 2:
            ref_subset_handle.write('>%s\n' % ref_seq_id)
            ref_subset_handle.write('%s\n' % ref_seq.seq)

            # add to ctg_ignore_region_dict_rd2
            if ref_seq_id in ctg_ignore_region_dict_rd1:
                ctg_ignore_region_dict_rd2[ref_seq_id] = ctg_ignore_region_dict_rd1[ref_seq_id]
        else:
            ref_seq_left_end_id = '%s_l' % ref_seq_id
            ref_seq_right_end_id = '%s_r' % ref_seq_id
            ref_seq_left_end = ref_seq.seq[:end_seq_len]
            ref_seq_right_end = ref_seq.seq[-end_seq_len:]

            # write out left end
            ref_subset_handle.write('>%s\n' % ref_seq_left_end_id)
            ref_subset_handle.write('%s\n' % ref_seq_left_end)

            # write out right end
            ref_subset_handle.write('>%s\n' % ref_seq_right_end_id)
            ref_subset_handle.write('%s\n' % ref_seq_right_end)

            # add to ctg_ignore_region_dict_rd2
            if ref_seq_id in ctg_ignore_region_dict_rd1:
                current_seq_to_ignore_ends = ctg_ignore_region_dict_rd1[ref_seq_id]
                for end_to_ignore in current_seq_to_ignore_ends:
                    if end_to_ignore == 'left_end':
                        ctg_ignore_region_dict_rd2[ref_seq_left_end_id] = {'left_end'}
                    if end_to_ignore == 'right_end':
                        ctg_ignore_region_dict_rd2[ref_seq_right_end_id] = {'right_end'}
    ref_subset_handle.close()

    return ctg_ignore_region_dict_rd2


combined_input_gnms  = '/Users/songweizhi/Desktop/3_Oral_refined_MAGs_combined.fa'
combined_barrnap_gff = '/Users/songweizhi/Desktop/combined_barrnap.gff'
combined_1st_round_unlinked_mags = ''
combined_1st_round_unlinked_mag_end_seq = ''
end_seq_len = ''


ctg_len_dict = {}
for each_ctg_record in SeqIO.parse(combined_input_gnms, 'fasta'):
    ctg_len_dict[each_ctg_record.id] = len(each_ctg_record.seq)
ctg_ignore_region_dict_rd1 = get_regions_to_ignore_from_barrnap_output(combined_barrnap_gff, ctg_len_dict)
ctg_ignore_region_dict_rd2 = get_unlinked_mag_end_seq(combined_1st_round_unlinked_mags, combined_1st_round_unlinked_mag_end_seq, end_seq_len, ctg_ignore_region_dict_rd1)











import os
from Bio import SeqIO, AlignIO


seq_in_16s      = '/Users/songweizhi/Desktop/00/GI_128_16S_0.999.fasta'
min_cov_id = 1
sliding_window = 25


seq_in_16s_aln          = '/Users/songweizhi/Desktop/00/GI_128_16S_0.999.fasta.aln'
seq_in_16s_filtered     = '/Users/songweizhi/Desktop/00/GI_128_16S_0.999_filtered.fasta'
alignment_file_out      = '/Users/songweizhi/Desktop/00/GI_128_16S_0.999_filtered.fasta.aln'


seq_in_16s_mafft_cmd = 'mafft --quiet --retree 1 %s > %s' % (seq_in_16s, seq_in_16s_aln)
os.system(seq_in_16s_mafft_cmd)

alignment_in = AlignIO.read(seq_in_16s_aln, "fasta")
total_col_num = alignment_in.get_alignment_length()
seq_id_list = [seq.id for seq in alignment_in]
seqs_to_remove = set()
col_index = 0
while col_index < (total_col_num - sliding_window + 1):
    current_column = [str(seg.seq) for seg in alignment_in[:, col_index:(col_index + sliding_window)]]
    non_dash_number = len(current_column) - current_column.count(sliding_window*'-')
    non_gap_percent = float("{0:.3f}".format(non_dash_number*100/len(seq_id_list)))
    if non_gap_percent < min_cov_id:
        row_index = 0
        for each_base in current_column:
            if each_base != (sliding_window*'-'):
                seqs_to_remove.add(seq_id_list[row_index])
            row_index += 1
    col_index += 1


seq_in_16s_filtered_handle = open(seq_in_16s_filtered, 'w')
for read_seq in SeqIO.parse(seq_in_16s, 'fasta'):
    if read_seq.id not in seqs_to_remove:
        seq_in_16s_filtered_handle.write('>%s\n' % read_seq.id)
        seq_in_16s_filtered_handle.write('%s\n' % str(read_seq.seq))
seq_in_16s_filtered_handle.close()


filtered_16s_mafft_cmd = 'mafft --quiet --retree 1 %s > %s' % (seq_in_16s_filtered, alignment_file_out)
os.system(filtered_16s_mafft_cmd)

print(seqs_to_remove)
print(len(seqs_to_remove))



#
# linkage_file       = '/Users/songweizhi/Desktop/assess_linkages_hc/hc_0512_tmp_stats_combined_filtered_with_assessment_ani97_imag99.5_i16S99.5.txt'
# clp_pct_ratio_file = '/Users/songweizhi/Desktop/assess_linkages_hc/rd1_clp_pct_diff.txt'
#
# filtered_marker_to_gnm_set = set()
# for each_link in open(linkage_file):
#     each_link_split = each_link.strip().split('\t')
#     marker_id = each_link_split[0]
#     genome_id = each_link_split[1]
#     marker_to_gnm_key = '%s___%s' % (marker_id, genome_id)
#     filtered_marker_to_gnm_set.add(marker_to_gnm_key)
#
#
# for each_link in open(clp_pct_ratio_file):
#     each_link_split = each_link.strip().split('\t')
#     marker_id = each_link_split[0]
#     genome_id = each_link_split[1]
#     marker_to_gnm_key = '%s___%s' % (marker_id, genome_id)
#     if marker_to_gnm_key in filtered_marker_to_gnm_set:
#         if genome_id in ['cami_hc_3', 'cami_hc_13', 'cami_hc_14', 'cami_hc_101', 'cami_hc_38', 'cami_hc_127', 'cami_hc_11']:
#             print(each_link.strip())
#             pass
#         else:
#             pass
#


reads_num = 315957101
subsample_rate = 10
reads_to_subsample = round(reads_num/subsample_rate)
print(reads_num/subsample_rate)
print(reads_to_subsample)